@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith8.cc,v 1.61 2009/01/16 18:18:58 sshwarts Exp $
+// $Id: arith8.cc,v 1.64 2010/03/05 08:54:06 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -251,7 +245,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 4
   Bit8u op1, op2, sum;
 
   /* XADD dst(r/m8), src(r8)
@@ -271,15 +264,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
 
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
-#else
-  BX_INFO(("XADD_EbGb: not supported on < 80486"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 4
   Bit8u op1, op2, sum;
 
   /* XADD dst(r/m8), src(r8)
@@ -300,10 +288,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
 
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
-#else
-  BX_INFO(("XADD_EbGb: not supported on < 80486"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EbIbM(bxInstruction_c *i)
@@ -470,7 +454,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EbR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbM(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 4
   Bit8u op1_8, op2_8, diff_8;
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
@@ -488,16 +471,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbM(bxInstruction_c *i)
     // accumulator <-- dest
     AL = op1_8;
   }
-
-#else
-  BX_INFO(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbR(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 4
   Bit8u op1_8, op2_8, diff_8;
 
   op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
@@ -513,9 +490,4 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbR(bxInstruction_c *i)
     // accumulator <-- dest
     AL = op1_8;
   }
-
-#else
-  BX_INFO(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
