@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit.cc,v 1.64 2009/11/02 15:00:47 sshwarts Exp $
+// $Id: bit.cc,v 1.68 2010/03/19 10:00:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -242,15 +236,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SETNLE_EbR(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
 }
 
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSWAP_RX(bxInstruction_c *i)
+{
+  BX_ERROR(("BSWAP with 16-bit opsize: undefined behavior !"));
+  BX_WRITE_16BIT_REG(i->opcodeReg(), 0);
+}
+
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSWAP_ERX(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL >= 4
   Bit32u val32, b0, b1, b2, b3;
 
-  if (i->os32L() == 0) {
-    BX_ERROR(("BSWAP with 16-bit opsize: undefined behavior !"));
-  }
-
   val32 = BX_READ_32BIT_REG(i->opcodeReg());
   b0 = val32 & 0xff; val32 >>= 8;
   b1 = val32 & 0xff; val32 >>= 8;
@@ -259,10 +254,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSWAP_ERX(bxInstruction_c *i)
   val32 = (b0<<24) | (b1<<16) | (b2<<8) | b3;
 
   BX_WRITE_32BIT_REGZ(i->opcodeReg(), val32);
-#else
-  BX_INFO(("BSWAP_ERX: required CPU >= 4, use --enable-cpu-level=4 option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 #if BX_SUPPORT_X86_64
@@ -285,12 +276,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSWAP_RRX(bxInstruction_c *i)
 }
 #endif
 
-// 3-byte opcodes
-#if (BX_SUPPORT_SSE >= 4) || (BX_SUPPORT_SSE >= 3 && BX_SUPPORT_SSE_EXTENSION > 0)
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GwEw(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit16u val16, b0, b1;
 
   if (i->modC0()) {
@@ -306,15 +293,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GwEw(bxInstruction_c *i)
   val16 = (b0<<8) | b1;
 
   BX_WRITE_16BIT_REG(i->nnn(), val16);
-#else
-  BX_INFO(("MOVBE_GwEw: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EwGw(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit16u val16 = BX_READ_16BIT_REG(i->nnn()), b0, b1;
 
   b0 = val16 & 0xff; val16 >>= 8;
@@ -328,15 +310,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EwGw(bxInstruction_c *i)
     bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     write_virtual_word(i->seg(), eaddr, val16);
   }
-#else
-  BX_INFO(("MOVBE_EwGw: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GdEd(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit32u val32, b0, b1, b2, b3;
 
   if (i->modC0()) {
@@ -354,15 +331,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GdEd(bxInstruction_c *i)
   val32 = (b0<<24) | (b1<<16) | (b2<<8) | b3;
 
   BX_WRITE_32BIT_REGZ(i->nnn(), val32);
-#else
-  BX_INFO(("MOVBE_GdEd: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EdGd(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit32u val32 = BX_READ_32BIT_REG(i->nnn()), b0, b1, b2, b3;
 
   b0 = val32 & 0xff; val32 >>= 8;
@@ -378,17 +350,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EdGd(bxInstruction_c *i)
     bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     write_virtual_dword(i->seg(), eaddr, val32);
   }
-#else
-  BX_INFO(("MOVBE_EdGd: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 #if BX_SUPPORT_X86_64
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GqEq(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit64u val64, b0, b1, b2, b3, b4, b5, b6, b7;
 
   if (i->modC0()) {
@@ -410,15 +377,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_GqEq(bxInstruction_c *i)
   val64 = (b0<<56) | (b1<<48) | (b2<<40) | (b3<<32) | (b4<<24) | (b5<<16) | (b6<<8) | b7;
 
   BX_WRITE_64BIT_REG(i->nnn(), val64);
-#else
-  BX_INFO(("MOVBE_GqEq: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EqGq(bxInstruction_c *i)
 {
-#if BX_SUPPORT_MOVBE
   Bit64u val64 = BX_READ_64BIT_REG(i->nnn());
   Bit64u b0, b1, b2, b3, b4, b5, b6, b7;
 
@@ -439,14 +401,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVBE_EqGq(bxInstruction_c *i)
     bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     write_virtual_qword_64(i->seg(), eaddr, val64);
   }
-#else
-  BX_INFO(("MOVBE_EqGq: required MOVBE support, use --enable-movbe option"));
-  exception(BX_UD_EXCEPTION, 0, 0);
-#endif
 }
 
-#endif // (BX_SUPPORT_X86_64)
+#endif // BX_SUPPORT_X86_64
 
-#endif // (BX_SUPPORT_SSE >= 4) || (BX_SUPPORT_SSE >= 3 && BX_SUPPORT_SSE_EXTENSION > 0)
-
-#endif // (BX_CPU_LEVEL >= 3)
+#endif // BX_CPU_LEVEL >= 3
