@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpuid.cc,v 1.86 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: cpuid.cc,v 1.114 2010/04/24 09:36:04 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2007-2009 Stanislav Shwartsman
+//   Copyright (c) 2007-2010 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -26,6 +26,8 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#include "param_names.h"
+
 #if BX_SUPPORT_X86_64==0
 // Make life easier for merging code.
 #define RAX EAX
@@ -34,7 +36,16 @@
 #define RDX EDX
 #endif
 
-/* Get CPU version information. */
+/*
+ * Get CPU version information:
+ *
+ * [3:0]   Stepping ID
+ * [7:4]   Model: starts at 1
+ * [11:8]  Family: 4=486, 5=Pentium, 6=PPro, ...
+ * [13:12] Type: 0=OEM, 1=overdrive, 2=dual cpu, 3=reserved
+ * [31:14] Reserved
+ */
+
 Bit32u BX_CPU_C::get_cpu_version_information(void)
 {
   Bit32u family = 0, model = 0, stepping = 0;
@@ -64,11 +75,12 @@ Bit32u BX_CPU_C::get_cpu_version_information(void)
 
 #elif BX_CPU_LEVEL == 5	
   family   = 5;
-#if BX_SUPPORT_MMX
-  model    = 4;         // Pentium MMX
-#else
-  model    = 1;         // Pentium 60/66
-#endif
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_MMX))
+    model = 4; // Pentium MMX
+  else
+    model = 1; // Pentium 60/66
+
   stepping = 3;
 
   /* ****** */
@@ -77,7 +89,10 @@ Bit32u BX_CPU_C::get_cpu_version_information(void)
 
 #elif BX_CPU_LEVEL == 6
 
-#if BX_SUPPORT_SSE >= 2 // Pentium 4 processor
+  unsigned sse_enabled = SIM->get_param_enum(BXPN_CPUID_SSE)->get();
+
+  if (sse_enabled >= 2) {
+
 /*
      The model, family, and processor type for the first
      processor in the Intel Pentium 4 family is as follows:
@@ -86,19 +101,20 @@ Bit32u BX_CPU_C::get_cpu_version_information(void)
 		* Processor Type-00B (OEM)
                 * Stepping-0B
 */
-  model    = 0;
-  family   = 0xf;
-  stepping = 0;
+    model    = 0;
+    family   = 0xf;
+    stepping = 0;
 
 #if BX_SUPPORT_X86_64
-  model    = 2;         // Hammer returns what?
+    model    = 2;       // Hammer returns what?
 #endif
 
-#else	                // Pentium Pro/Pentium II/Pentium III processor
-  family   = 6;
-  model    = 8;
-  stepping = 3;
-#endif
+  }
+  else {                // Pentium Pro/Pentium II/Pentium III processor
+    family   = 6;
+    model    = 8;
+    stepping = 3;
+  }
 
 #else
   BX_PANIC(("CPUID family ID not implemented for CPU LEVEL > 6"));
@@ -117,20 +133,21 @@ Bit32u BX_CPU_C::get_extended_cpuid_features(void)
 {
   // [0:0]   SSE3: SSE3 Instructions
   // [1:1]   PCLMULQDQ Instruction support
-  // [2:2]   reserved
+  // [2:2]   DTES64: 64-bit DS area
   // [3:3]   MONITOR/MWAIT support
   // [4:4]   DS-CPL: CPL qualified debug store
   // [5:5]   VMX: Virtual Machine Technology
   // [6:6]   SMX: Secure Virtual Machine Technology
   // [7:7]   EST: Enhanced Intel SpeedStep Technology
   // [8:8]   TM2: Thermal Monitor 2
-  // [9:9]   SSE3E: SSE3E Instructions (Intel Core Duo 2 new instructions)
+  // [9:9]   SSSE3: SSSE3 Instructions
   // [10:10] CNXT-ID: L1 context ID
   // [11:11] reserved
   // [12:12] FMA Instructions support
   // [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   // [14:14] xTPR update control
-  // [17:15] reserved
+  // [16:15] reserved
+  // [17:17] PCID: Process Context Identifiers
   // [18:18] DCA - Direct Cache Access
   // [19:19] SSE4.1 Instructions
   // [20:20] SSE4.2 Instructions
@@ -146,46 +163,53 @@ Bit32u BX_CPU_C::get_extended_cpuid_features(void)
 
   Bit32u features = 0;
 
-#if BX_SUPPORT_SSE >= 3
-  features |= 0x1;               // support SSE3
-#endif
-#if BX_SUPPORT_MONITOR_MWAIT
-  features |= (1<<3);            // support MONITOR/MWAIT
-#endif
-#if BX_SUPPORT_VMX
-  features |= (1<<5);            // support VMX
-#endif
-#if (BX_SUPPORT_SSE >= 4) || (BX_SUPPORT_SSE >= 3 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<9);            // support SSE3E
-#endif
+  // support SSE3
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE3))
+    features |= (1<<0);
+
+  // support for PCLMULQDQ
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_AES_PCLMULQDQ))
+    features |= (1<<1);
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_MONITOR_MWAIT))
+    features |= (1<<3);
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_VMX))
+    features |= (1<<5);
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSSE3))
+    features |= (1<<9);
 
 #if BX_SUPPORT_X86_64
-  features |= (1<<13);           // support CMPXCHG16B
+  // support CMPXCHG16B
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_X86_64))
+    features |= (1<<13);
 #endif
 
-#if BX_SUPPORT_SSE >= 4
-  features |= (1<<19);           // support SSE4.1
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE4_1))
+    features |= (1<<19);
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE4_2))
+    features |= (1<<20);
+
+#if BX_SUPPORT_X2APIC
+  features |= (1<<21);           // support X2APIC
 #endif
 
-#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<20);           // support SSE4.2
-#endif
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_MOVBE))
+    features |= (1<<22);
 
-#if BX_SUPPORT_MOVBE
-  features |= (1<<22);           // support MOVBE instruction
-#endif
+  // enable POPCNT if SSE4_2 is enabled
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE4_2))
+    features |= (1<<23);
 
-#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<23);           // support POPCNT instruction
-#endif
+  // support for AES
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_AES_PCLMULQDQ))
+    features |= (1<<25);
 
-#if BX_SUPPORT_AES
-  features |= (1<<25);           // support AES instructions
-#endif
-
-#if BX_SUPPORT_XSAVE
-  features |= (1<<26) | (1<<27); // support XSAVE extensions
-#endif
+  // support XSAVE extensions
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_XSAVE))
+    features |= (1<<26) | (1<<27);
 
   return features;
 }
@@ -211,7 +235,7 @@ Bit32u BX_CPU_C::get_std_cpuid_features(void)
   //   [15:15] CMOV: Cond Mov/Cmp Instructions
   //   [16:16] PAT: Page Attribute Table
   //   [17:17] PSE-36: Physical Address Extensions
-  //   [18:18] Processor Serial Number
+  //   [18:18] PSN: Processor Serial Number
   //   [19:19] CLFLUSH: CLFLUSH Instruction support
   //   [20:20] Reserved
   //   [21:21] DS: Debug Store
@@ -220,7 +244,7 @@ Bit32u BX_CPU_C::get_std_cpuid_features(void)
   //   [24:24] FXSR: FXSAVE/FXRSTOR (also indicates CR4.OSFXSR is available)
   //   [25:25] SSE: SSE Extensions
   //   [26:26] SSE2: SSE2 Extensions
-  //   [27:27] Reserved
+  //   [27:27] Self Snoop
   //   [28:28] Hyper Threading Technology
   //   [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
@@ -228,22 +252,23 @@ Bit32u BX_CPU_C::get_std_cpuid_features(void)
 
   Bit32u features = 0;
 
-#if BX_SUPPORT_FPU
-  features |= (1<<0);
-#endif
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_X87))
+    features |= (1<<0);
+
 #if BX_CPU_LEVEL >= 5
-  features |= (1<<1);
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_PENTIUM)) {
+    // Pentium only features
+    features |= (1<<1);             // support VME
+    features |= (1<<3);             // support PSE
+    features |= (1<<4);             // support Time Stamp Counter
+    features |= (1<<5);             // support RDMSR/WRMSR
+    features |= (1<<7) | (1<<14);   // support Machine Check
+    features |= (1<<8);             // support CMPXCHG8B instruction
+  }
 #endif
 
   features |= (1<<2);   // support Debug Extensions
 
-#if BX_CPU_LEVEL >= 5
-  features |= (1<<4);             // support Time Stamp Counter
-  features |= (1<<5);             // support RDMSR/WRMSR
-  features |= (1<<7) | (1<<14);   // support Machine Check
-  features |= (1<<8);             // support CMPXCHG8B instruction
-#endif
-
 #if BX_SUPPORT_APIC
   // if MSR_APICBASE APIC Global Enable bit has been cleared,
   // the CPUID feature flag for the APIC is set to 0.
@@ -251,42 +276,35 @@ Bit32u BX_CPU_C::get_std_cpuid_features(void)
     features |= (1<<9); // APIC on chip
 #endif
 
-#if BX_SUPPORT_SEP
-  features |= (1<<11);  // SYSENTER/SYSEXIT
-#endif
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SYSENTER_SYSEXIT))
+    features |= (1<<11);
 
-#if BX_SUPPORT_CLFLUSH
-  features |= (1<<19);  // Implement CLFLUSH instruction
-#endif
-
-#if BX_SUPPORT_MMX
-  features |= (1<<23);  // support MMX
-#endif
-
-#if BX_CPU_LEVEL >= 6
-  features |= (1<<24);  // Implement FSAVE/FXRSTOR instructions.
-#endif
-
-#if BX_SUPPORT_SSE >= 1
-  features |= (1<<25);  // support SSE
-#endif
-#if BX_SUPPORT_SSE >= 2
-  features |= (1<<26);  // support SSE2
-#endif
-
-#if BX_CPU_LEVEL >= 6
-  features |= (1<<15);  // Implement CMOV instructions.
-#endif
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_CLFLUSH))
+    features |= (1<<19);
 
 #if BX_CPU_LEVEL >= 5
-  features |= (1<<3);   // support PSE
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_MMX))
+    features |= (1<<23);
 #endif
+
 #if BX_CPU_LEVEL >= 6
-  features |= (1<<6);   // support PAE
-  features |= (1<<12);  // support MTRRs
-  features |= (1<<13);  // support Global pages
-  features |= (1<<16);  // support PAT
-  features |= (1<<17);  // support PSE-36
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_P6)) {
+    features |= (1<<6);   // support PAE
+    features |= (1<<12);  // support MTRRs
+    features |= (1<<13);  // support Global pages
+    features |= (1<<15);  // support CMOV instructions
+    features |= (1<<16);  // support PAT
+    features |= (1<<17);  // support PSE-36
+  }
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_FXSAVE_FXRSTOR))
+    features |= (1<<24);  // support FSAVE/FXRSTOR instructions
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE))
+    features |= (1<<25);
+
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE2))
+    features |= (1<<26);
 #endif
 
 #if BX_SUPPORT_SMP
@@ -302,9 +320,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CPUID(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit32u function    = EAX;
-#if BX_SUPPORT_XSAVE
   Bit32u subfunction = ECX;
-#endif
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
@@ -313,58 +329,46 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CPUID(bxInstruction_c *i)
   }
 #endif
 
+  unsigned max_std_function = BX_CPU_THIS_PTR cpuid_std_function[0].eax;
+  unsigned max_ext_function = BX_CPU_THIS_PTR cpuid_ext_function[0].eax;
+
   if(function < 0x80000000) {
-    if(function < MAX_STD_CPUID_FUNCTION) {
+    if(function <= max_std_function) {
+#if BX_SUPPORT_X2APIC
+      if (function == 0xb) {
+        bx_cpuid_extended_topology_leaf(subfunction);
+        return;
+      }
+#endif
+#if BX_CPU_LEVEL >= 6
+      if (function == 0xd) {
+        bx_cpuid_xsave_leaf(subfunction);
+        return;
+      }
+#endif
       RAX = BX_CPU_THIS_PTR cpuid_std_function[function].eax;
       RBX = BX_CPU_THIS_PTR cpuid_std_function[function].ebx;
       RCX = BX_CPU_THIS_PTR cpuid_std_function[function].ecx;
       RDX = BX_CPU_THIS_PTR cpuid_std_function[function].edx;
-#if BX_SUPPORT_APIC
-      if (function == 1) {
-        // if MSR_APICBASE APIC Global Enable bit has been cleared,
-        // the CPUID feature flag for the APIC is set to 0.
-        if ((BX_CPU_THIS_PTR msr.apicbase & 0x800) == 0)
-          RDX &= ~(1<<9); // APIC on chip
-      }
-#endif
-#if BX_SUPPORT_XSAVE
-      if (function == 0xD && subfunction > 0) {
-        RAX = 0;
-        RBX = 0;
-        RCX = 0;
-        RDX = 0;
-      }
-#endif
       return;
     }
   }
   else {
-    function -= 0x80000000;
-    if(function < MAX_EXT_CPUID_FUNCTION) {
+    if(function <= max_ext_function) {
+      function -= 0x80000000;
       RAX = BX_CPU_THIS_PTR cpuid_ext_function[function].eax;
       RBX = BX_CPU_THIS_PTR cpuid_ext_function[function].ebx;
       RCX = BX_CPU_THIS_PTR cpuid_ext_function[function].ecx;
       RDX = BX_CPU_THIS_PTR cpuid_ext_function[function].edx;
-#if BX_SUPPORT_APIC
-      if (function == 1) {
-        // if MSR_APICBASE APIC Global Enable bit has been cleared,
-        // the CPUID feature flag for the APIC is set to 0.
-        if ((BX_CPU_THIS_PTR msr.apicbase & 0x800) == 0)
-          RDX &= ~(1<<9); // APIC on chip
-      }
-#endif
       return;
     }
   }
 
-  // unknown CPUID function
-  RAX = 0;
-  RBX = 0;
-  RCX = 0;
-  RDX = 0;
-#else
-  BX_INFO(("CPUID: not available on < 486"));
-  exception(BX_UD_EXCEPTION, 0, 0);
+  // unknown CPUID function - return maximum standard leaf
+  RAX = BX_CPU_THIS_PTR cpuid_std_function[max_std_function].eax;
+  RBX = BX_CPU_THIS_PTR cpuid_std_function[max_std_function].ebx;
+  RCX = BX_CPU_THIS_PTR cpuid_std_function[max_std_function].ecx;
+  RDX = BX_CPU_THIS_PTR cpuid_std_function[max_std_function].edx;
 #endif
 }
 
@@ -373,6 +377,9 @@ void BX_CPU_C::set_cpuid_defaults(void)
   Bit8u *vendor_string = (Bit8u *)SIM->get_param_string(BXPN_VENDOR_STRING)->getptr();
   Bit8u *brand_string = (Bit8u *)SIM->get_param_string(BXPN_BRAND_STRING)->getptr();
   bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
+#if BX_SUPPORT_X86_64
+  bx_bool xlarge_pages_enabled = SIM->get_param_bool(BXPN_CPUID_1G_PAGES)->get();
+#endif
 
   cpuid_function_t *cpuid;
   int i;
@@ -405,22 +412,28 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->eax = 1;
 #else
   // for Pentium Pro, Pentium II, Pentium 4 processors
-  cpuid->eax = 2;
+  cpuid->eax = 3;
+
   // do not report CPUID functions above 0x3 if cpuid_limit_winnt is set
   // to workaround WinNT issue.
   if (! cpuid_limit_winnt) {
     if (BX_SUPPORT_MONITOR_MWAIT)
       cpuid->eax = 0x5;
-    if (BX_SUPPORT_XSAVE)
-      cpuid->eax = 0xD;
+#if BX_SUPPORT_X2APIC
+    cpuid->eax = 0xb;
+#endif
+    if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_XSAVE))
+      cpuid->eax = 0xd;
   }
 #endif
 
   // CPUID vendor string (e.g. GenuineIntel, AuthenticAMD, CentaurHauls, ...)
-  memcpy(&(cpuid->ebx), vendor_string    , 4);
+  memcpy(&(cpuid->ebx), vendor_string,     4);
   memcpy(&(cpuid->edx), vendor_string + 4, 4);
   memcpy(&(cpuid->ecx), vendor_string + 8, 4);
 
+  BX_INFO(("CPUID[0x00000000]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // ------------------------------------------------------
   // CPUID function 0x00000001
   cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[1]);
@@ -440,35 +453,34 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //   [31:24] Local Apic ID
 
   cpuid->ebx = 0;
-#if BX_SUPPORT_APIC
-  cpuid->ebx |= ((BX_CPU_THIS_PTR lapic.get_id() & 0xff) << 24);
-#endif
-#if BX_SUPPORT_CLFLUSH
-  cpuid->ebx |= (CACHE_LINE_SIZE / 8) << 8;
-#endif
-#if BX_SUPPORT_SMP
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_CLFLUSH)) {
+    cpuid->ebx |= (CACHE_LINE_SIZE / 8) << 8;
+  }
   unsigned n_logical_processors = SIM->get_param_num(BXPN_CPU_NCORES)->get()*SIM->get_param_num(BXPN_CPU_NTHREADS)->get();
   if (n_logical_processors > 1)
     cpuid->ebx |= (n_logical_processors << 16);
+#if BX_SUPPORT_APIC
+  cpuid->ebx |= ((BX_CPU_THIS_PTR lapic.get_id() & 0xff) << 24);
 #endif
 
   // ECX:       Extended Feature Flags
   //   [0:0]   SSE3: SSE3 Instructions
   //   [1:1]   PCLMULQDQ Instruction support
-  //   [2:2]   reserved
+  //   [2:2]   DTES64: 64-bit DS area
   //   [3:3]   MONITOR/MWAIT support
   //   [4:4]   DS-CPL: CPL qualified debug store
   //   [5:5]   VMX: Virtual Machine Technology
   //   [6:6]   SMX: Secure Virtual Machine Technology
   //   [7:7]   EST: Enhanced Intel SpeedStep Technology
   //   [8:8]   TM2: Thermal Monitor 2
-  //   [9:9]   SSE3E: SSE3E Instructions (Intel Core Duo 2 new instructions)
+  //   [9:9]   SSSE3: SSSE3 Instructions
   //   [10:10] CNXT-ID: L1 context ID
   //   [11:11] reserved
   //   [12:12] FMA Instructions support
   //   [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   //   [14:14] xTPR update control
-  //   [17:15] reserved
+  //   [16:15] reserved
+  //   [17:17] PCID: Process Context Identifiers
   //   [18:18] DCA - Direct Cache Access
   //   [19:19] SSE4.1 Instructions
   //   [20:20] SSE4.2 Instructions
@@ -483,7 +495,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //   [31:29] reserved
   cpuid->ecx = get_extended_cpuid_features();
 
-  // EDX:       Feature Flags
+  // EDX:       Standard Feature Flags
   //   [0:0]   FPU on chip
   //   [1:1]   VME: Virtual-8086 Mode enhancements
   //   [2:2]   DE: Debug Extensions (I/O breakpoints)
@@ -502,7 +514,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //   [15:15] CMOV: Cond Mov/Cmp Instructions
   //   [16:16] PAT: Page Attribute Table
   //   [17:17] PSE-36: Physical Address Extensions
-  //   [18:18] Processor Serial Number
+  //   [18:18] PSN: Processor Serial Number
   //   [19:19] CLFLUSH: CLFLUSH Instruction support
   //   [20:20] Reserved
   //   [21:21] DS: Debug Store
@@ -511,13 +523,15 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //   [24:24] FXSR: FXSAVE/FXRSTOR (also indicates CR4.OSFXSR is available)
   //   [25:25] SSE: SSE Extensions
   //   [26:26] SSE2: SSE2 Extensions
-  //   [27:27] Reserved
+  //   [27:27] Self Snoop
   //   [28:28] Hyper Threading Technology
   //   [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
   //   [31:31] PBE: Pending Break Enable
   cpuid->edx = get_std_cpuid_features();
 
+  BX_INFO(("CPUID[0x00000001]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
 #if BX_CPU_LEVEL >= 6
   // ------------------------------------------------------
   // CPUID function 0x00000002 - Cache and TLB Descriptors
@@ -535,6 +549,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->edx = 0;
 #endif
 
+  BX_INFO(("CPUID[0x00000002]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // ------------------------------------------------------
   // CPUID function 0x00000003 - Processor Serial Number
   cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[3]);
@@ -544,6 +560,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0;
   cpuid->edx = 0;
 
+  BX_INFO(("CPUID[0x00000003]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // do not report CPUID functions above 0x3 if cpuid_limit_winnt is set
   // to workaround WinNT issue.
   if (cpuid_limit_winnt) return;
@@ -557,6 +575,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0;
   cpuid->edx = 0;
 
+  BX_INFO(("CPUID[0x00000004]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
 #if BX_SUPPORT_MONITOR_MWAIT
   // ------------------------------------------------------
   // CPUID function 0x00000005
@@ -573,24 +593,31 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ebx = CACHE_LINE_SIZE;
   cpuid->ecx = 3;
   cpuid->edx = 0;
+
+  BX_INFO(("CPUID[0x00000005]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
 #endif
 
-#if BX_SUPPORT_XSAVE
   // ------------------------------------------------------
-  // CPUID function 0x0000000D
-  cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[0xD]);
+  // CPUID function 0x0000000d
+  if (BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_XSAVE))
+  {
+    cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[0xd]);
 
-  // EAX - XCR0 lower 32 bits
-  // EBX - Maximum size (in bytes) required by enabled features
-  // ECX - Maximum size (in bytes) required by CPU supported features
-  // EDX - XCR0 upper 32 bits
-  cpuid->eax = BX_CPU_THIS_PTR xcr0.get32();
-  cpuid->ebx = 512+64;
-  cpuid->ecx = 512+64;
-  cpuid->edx = 0;
-#endif
+    // EAX - XCR0 lower 32 bits
+    // EBX - Maximum size (in bytes) required by enabled features
+    // ECX - Maximum size (in bytes) required by CPU supported features
+    // EDX - XCR0 upper 32 bits
+    cpuid->eax = BX_CPU_THIS_PTR xcr0.get32();
+    cpuid->ebx = 512+64;
+    cpuid->ecx = 512+64;
+    cpuid->edx = 0;
 
-#if BX_SUPPORT_SSE >= 2   // report Pentium 4 extended functions
+    BX_INFO(("CPUID[0x0000000D]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+  }
+
+  // do not report Pentium 4 extended functions if not needed
+  if (! BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_SSE2))
+    return;
 
   // ------------------------------------------------------
   // CPUID function 0x80000000
@@ -606,11 +633,13 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->edx = 0;          // Reserved for Intel
   cpuid->ecx = 0;
 #else
-  memcpy(&(cpuid->ebx), vendor_string    , 4);
+  memcpy(&(cpuid->ebx), vendor_string,     4);
   memcpy(&(cpuid->edx), vendor_string + 4, 4);
   memcpy(&(cpuid->ecx), vendor_string + 8, 4);
 #endif
 
+  BX_INFO(("CPUID[0x80000000]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // ------------------------------------------------------
   // CPUID function 0x80000001
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[1]);
@@ -682,12 +711,13 @@ void BX_CPU_C::set_cpuid_defaults(void)
 #endif
 #if BX_SUPPORT_X86_64
   features |= (1 << 29) | (1 << 27) | (1 << 25) | (1 << 20) | (1 << 11);
-#if BX_SUPPORT_1G_PAGES
-  features |= (1 << 26);
-#endif
+  if (xlarge_pages_enabled)
+    features |= (1 << 26);
 #endif
   cpuid->edx = features;
 
+  BX_INFO(("CPUID[0x80000001]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // Processor Brand String, use the value that is returned
   // by the first processor in the Pentium 4 family
   // (according to Intel manual or the AMD manual)
@@ -701,6 +731,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   memcpy(&(cpuid->ecx), brand_string +  8, 4);
   memcpy(&(cpuid->edx), brand_string + 12, 4);
 
+  BX_INFO(("CPUID[0x80000002]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // CPUID function 0x80000003
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[3]);
   memcpy(&(cpuid->eax), brand_string + 16, 4);
@@ -708,6 +740,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   memcpy(&(cpuid->ecx), brand_string + 24, 4);
   memcpy(&(cpuid->edx), brand_string + 28, 4);
 
+  BX_INFO(("CPUID[0x80000003]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // CPUID function 0x80000004
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[4]);
   memcpy(&(cpuid->eax), brand_string + 32, 4);
@@ -715,9 +749,13 @@ void BX_CPU_C::set_cpuid_defaults(void)
   memcpy(&(cpuid->ecx), brand_string + 40, 4);
   memcpy(&(cpuid->edx), brand_string + 44, 4);
 
+  BX_INFO(("CPUID[0x80000004]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
 #if BX_SUPPORT_X86_64
   // ------------------------------------------------------
   // CPUID function 0x80000005
+
+#if BX_CPU_VENDOR_INTEL == 0
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[5]);
 
   /* cache info (L1 cache) */
@@ -726,6 +764,9 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0x40020140;
   cpuid->edx = 0x40020140;
 
+  BX_INFO(("CPUID[0x80000005]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+#endif
+
   // ------------------------------------------------------
   // CPUID function 0x80000006
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[6]);
@@ -736,6 +777,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0x02008140;
   cpuid->edx = 0;
 
+  BX_INFO(("CPUID[0x80000006]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // ------------------------------------------------------
   // CPUID function 0x00000007
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[7]);
@@ -745,6 +788,8 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0;
   cpuid->edx = 0;
 
+  BX_INFO(("CPUID[0x80000007]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
+
   // ------------------------------------------------------
   // CPUID function 0x80000008
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[8]);
@@ -755,9 +800,295 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0; // Reserved, undefined
   cpuid->edx = 0;
 
-#endif // BX_SUPPORT_X86_64
+  BX_INFO(("CPUID[0x80000008]: %08x %08x %08x %08x", cpuid->eax, cpuid->ebx, cpuid->ecx, cpuid->edx));
 
-#endif // BX_SUPPORT_SSE >= 2
+#endif // BX_SUPPORT_X86_64
 
 #endif // BX_CPU_LEVEL >= 6
 }
+
+#if BX_SUPPORT_X2APIC
+
+#include <math.h>
+
+void BX_CPU_C::bx_cpuid_extended_topology_leaf(Bit32u subfunction)
+{
+  static int nthreads = SIM->get_param_num(BXPN_CPU_NTHREADS)->get();
+  static int ncores = SIM->get_param_num(BXPN_CPU_NCORES)->get();
+  static int nprocessors = SIM->get_param_num(BXPN_CPU_NPROCESSORS)->get();
+
+  switch(subfunction) {
+  case 0:
+     if (nthreads > 1) {
+        RAX = (bx_address) ceil(log(nthreads)/log(2));
+        RBX = nthreads;
+        RCX = subfunction | (1<<8);
+     }
+     else if (ncores > 1) {
+        RAX = (bx_address) ceil(log(ncores)/log(2));
+        RBX = ncores;
+        RCX = subfunction | (2<<8);
+     }
+     else if (nprocessors > 1) {
+        RAX = (bx_address) ceil(log(nprocessors)/log(2));
+        RBX = nprocessors;
+        RCX = subfunction;
+     }
+     else {
+        RAX = 0;
+        RBX = 0;
+        RCX = subfunction;
+     }
+     break;
+
+  case 1:
+     if (nthreads > 1) {
+        if (ncores > 1) {
+           RAX = (bx_address) ceil(log(ncores)/log(2));
+           RBX = ncores;
+           RCX = subfunction | (2<<8);
+        }
+        else if (nprocessors > 1) {
+           RAX = (bx_address) ceil(log(nprocessors)/log(2));
+           RBX = nprocessors;
+           RCX = subfunction;
+        }
+        else {
+           RAX = 0;
+           RBX = 0;
+           RCX = subfunction;
+        }
+     }
+     else if (ncores > 1) {
+        if (nprocessors > 1) {
+           RAX = (bx_address) ceil(log(nprocessors)/log(2));
+           RBX = nprocessors;
+           RCX = subfunction;
+        }
+        else {
+           RAX = 0;
+           RBX = 0;
+           RCX = subfunction;
+        }
+     } else {
+        RAX = 0;
+        RBX = 0;
+        RCX = subfunction;
+     }
+     break;
+
+  case 2:
+     if (nthreads > 1) {
+        if (nprocessors > 1) {
+           RAX = (bx_address) ceil(log(nprocessors)/log(2));
+           RBX = nprocessors;
+        }
+        else {
+           RAX = 0;
+           RBX = 0;
+        }
+     }
+     else {
+        RAX = 0;
+        RBX = 0;
+     }
+     RCX = subfunction;
+     break;
+
+   default:
+     RAX = 0;
+     RBX = 0;
+     RCX = subfunction;
+     break;
+   }
+
+   RDX = BX_CPU_THIS_PTR lapic.get_id(); // x2apic ID
+}
+
+#endif
+
+#if BX_CPU_LEVEL >= 6
+void BX_CPU_C::bx_cpuid_xsave_leaf(Bit32u subfunction)
+{
+  BX_ASSERT(BX_CPU_SUPPORT_ISA_EXTENSION(BX_CPU_XSAVE));
+  if (subfunction == 0) {
+    RAX = BX_CPU_THIS_PTR cpuid_std_function[0xd].eax;
+    RBX = BX_CPU_THIS_PTR cpuid_std_function[0xd].ebx;
+    RCX = BX_CPU_THIS_PTR cpuid_std_function[0xd].ecx;
+    RDX = BX_CPU_THIS_PTR cpuid_std_function[0xd].edx;
+  }
+  else {
+    RAX = 0; // reserved
+    RBX = 0; // reserved
+    RCX = 0; // reserved
+    RDX = 0; // reserved
+  }
+}
+#endif
+
+void BX_CPU_C::init_isa_features_bitmask(void)
+{
+  Bit32u features_bitmask = 0;
+
+  bx_bool mmx_enabled = 0, movbe_enabled = 0;
+  bx_bool sep_enabled = 0, xsave_enabled = 0;
+  bx_bool aes_enabled = 0, xapic_enabled = 0;
+  unsigned sse_enabled = 0;
+
+#if BX_CPU_LEVEL >= 5
+  mmx_enabled = SIM->get_param_bool(BXPN_CPUID_MMX)->get();
+#endif
+#if BX_CPU_LEVEL >= 6
+  sep_enabled = SIM->get_param_bool(BXPN_CPUID_SEP)->get();
+  aes_enabled = SIM->get_param_bool(BXPN_CPUID_AES)->get();
+  movbe_enabled = SIM->get_param_bool(BXPN_CPUID_MOVBE)->get();
+  xsave_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVE)->get();
+  xapic_enabled = SIM->get_param_bool(BXPN_CPUID_XAPIC)->get();
+  sse_enabled = SIM->get_param_enum(BXPN_CPUID_SSE)->get();
+#endif
+
+  // sanity checks
+#if BX_SUPPORT_3DNOW
+  if (! mmx_enabled) {
+    BX_PANIC(("PANIC: 3DNOW emulation requires MMX support !"));
+    return;
+  }
+#endif
+
+  if (aes_enabled) {
+     // AES required 3-byte opcode (SSS3E support or more)
+     if (sse_enabled < BX_CPUID_SUPPORT_SSSE3) {
+       BX_PANIC(("PANIC: AES support requires SSSE3 or higher !"));
+       return;
+     }
+  }
+
+  if (movbe_enabled) {
+     // MOVBE required 3-byte opcode (SSS3E support or more)
+     if (sse_enabled < BX_CPUID_SUPPORT_SSSE3) {
+       BX_PANIC(("PANIC: MOVBE support requires SSSE3 or higher !"));
+       return;
+     }
+  }
+
+  if (xapic_enabled) {
+     if (! BX_SUPPORT_APIC) {
+       BX_PANIC(("PANIC: XAPIC enabled when APIC is not compiled in !"));
+       return;
+     }
+  }
+
+  if (sse_enabled) {
+     if (mmx_enabled == 0 || BX_CPU_LEVEL < 6) {
+       BX_PANIC(("PANIC: SSE support requires P6 emulation with MMX enabled !"));
+       return;
+     }
+
+     if (sse_enabled >= BX_CPUID_SUPPORT_SSE2) {
+       if (! xapic_enabled) {
+         BX_PANIC(("PANIC: SSE2 is enabled and without XAPIC !"));
+         return;
+       }
+     }
+  }
+  else {
+     if (xsave_enabled) {
+       BX_PANIC(("PANIC: XSAVE emulation requires SSE support !"));
+       return;
+     }
+  }
+
+#if BX_SUPPORT_X86_64
+  if (sse_enabled < BX_CPUID_SUPPORT_SSE2) {
+    BX_PANIC(("PANIC: x86-64 emulation requires SSE2 support !"));
+    return;
+  }
+  if (! xapic_enabled) {
+    BX_PANIC(("PANIC: x86-64 emulation requires XAPIC support !"));
+    return;
+  }
+  if (! sep_enabled) {
+    BX_PANIC(("PANIC: x86-64 emulation requires SYSENTER/SYSEXIT support !"));
+    return;
+  }
+#endif
+
+#if BX_SUPPORT_VMX
+  if (! sep_enabled) {
+    BX_PANIC(("PANIC: VMX emulation requires SYSENTER/SYSEXIT support !"));
+    return;
+  }
+#endif
+
+#if BX_SUPPORT_FPU
+  features_bitmask |= BX_CPU_X87;
+#endif
+#if BX_CPU_LEVEL >= 4
+  features_bitmask |= BX_CPU_486;
+#endif
+#if BX_CPU_LEVEL >= 5
+  features_bitmask |= BX_CPU_PENTIUM;
+
+  if (mmx_enabled)
+    features_bitmask |= BX_CPU_MMX;
+#endif
+#if BX_SUPPORT_3DNOW
+  features_bitmask |= BX_CPU_3DNOW;
+#endif
+#if BX_SUPPORT_MONITOR_MWAIT
+  features_bitmask |= BX_CPU_MONITOR_MWAIT;
+#endif
+
+#if BX_CPU_LEVEL >= 6
+  features_bitmask |= BX_CPU_P6;
+
+  // FXSAVE/FXRSTOR support come with Pentium II
+  if (mmx_enabled)
+    features_bitmask |= BX_CPU_FXSAVE_FXRSTOR;
+
+  // enabled CLFLUSH only when SSE2 or higher is enabled
+  if (sse_enabled >= BX_CPUID_SUPPORT_SSE2)
+    features_bitmask |= BX_CPU_CLFLUSH;
+
+  // determine SSE in runtime
+  switch (sse_enabled) {
+    case BX_CPUID_SUPPORT_SSE4_2:
+      features_bitmask |= BX_CPU_SSE4_2;
+    case BX_CPUID_SUPPORT_SSE4_1:
+      features_bitmask |= BX_CPU_SSE4_1;
+    case BX_CPUID_SUPPORT_SSSE3:
+      features_bitmask |= BX_CPU_SSSE3;
+    case BX_CPUID_SUPPORT_SSE3:
+      features_bitmask |= BX_CPU_SSE3;
+    case BX_CPUID_SUPPORT_SSE2:
+      features_bitmask |= BX_CPU_SSE2;
+    case BX_CPUID_SUPPORT_SSE:
+      features_bitmask |= BX_CPU_SSE;
+    case BX_CPUID_SUPPORT_NOSSE:
+    default:
+      break;
+  };
+
+  if (sep_enabled)
+    features_bitmask |= BX_CPU_SYSENTER_SYSEXIT;
+
+  if (xsave_enabled)
+    features_bitmask |= BX_CPU_XSAVE;
+
+  if (aes_enabled)
+    features_bitmask |= BX_CPU_AES_PCLMULQDQ;
+
+  if (movbe_enabled)
+    features_bitmask |= BX_CPU_MOVBE;
+#endif
+
+#if BX_SUPPORT_VMX
+  features_bitmask |= BX_CPU_VMX;
+#endif
+
+#if BX_SUPPORT_X86_64
+  features_bitmask |= BX_CPU_X86_64;
+#endif
+
+  BX_CPU_THIS_PTR isa_extensions_bitmask = features_bitmask;
+}
