@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ctrl_xfer16.cc,v 1.68 2009/03/22 21:12:35 sshwarts Exp $
+// $Id: ctrl_xfer16.cc,v 1.72 2010/03/14 15:51:26 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -44,7 +38,7 @@ BX_CPP_INLINE void BX_CPP_AttrRegparmN(1) BX_CPU_C::branch_near16(Bit16u new_IP)
   if (new_IP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled)
   {
     BX_ERROR(("branch_near16: offset outside of CS limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   EIP = new_IP;
@@ -70,7 +64,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETnear16_Iw(bxInstruction_c *i)
   if (return_IP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled)
   {
     BX_ERROR(("RETnear16_Iw: IP > limit"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   EIP = return_IP;
@@ -102,7 +96,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETnear16(bxInstruction_c *i)
   if (return_IP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled)
   {
     BX_ERROR(("RETnear16: IP > limit"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   EIP = return_IP;
@@ -124,33 +118,34 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETfar16_Iw(bxInstruction_c *i)
 
   Bit16s imm16 = (Bit16s) i->Iw();
 
-  RSP_SPECULATIVE;
-
   if (protected_mode()) {
     return_protected(i, imm16);
     goto done;
   }
 
+  RSP_SPECULATIVE;
+
   ip     = pop_16();
   cs_raw = pop_16();
 
   // CS.LIMIT can't change when in real/v8086 mode
   if (ip > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("RETfar16_Iw: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
-  RIP = (Bit32u) ip;
+  EIP = (Bit32u) ip;
 
   if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.d_b)
     ESP += imm16;
   else
      SP += imm16;
 
-done:
   RSP_COMMIT;
 
+done:
+
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_RET,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, EIP);
 }
@@ -165,28 +160,29 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETfar16(bxInstruction_c *i)
   BX_CPU_THIS_PTR show_flag |= Flag_ret;
 #endif
 
-  RSP_SPECULATIVE;
-
   if (protected_mode()) {
     return_protected(i, 0);
     goto done;
   }
 
+  RSP_SPECULATIVE;
+
   ip     = pop_16();
   cs_raw = pop_16();
 
   // CS.LIMIT can't change when in real/v8086 mode
   if (ip > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("RETfar16: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
   EIP = (Bit32u) ip;
 
-done:
   RSP_COMMIT;
 
+done:
+
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_RET,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, EIP);
 }
@@ -236,7 +232,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL16_Ap(bxInstruction_c *i)
   // CS.LIMIT can't change when in real/v8086 mode
   if (disp16 > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("CALL16_Ap: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
@@ -298,7 +294,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL16_Ep(bxInstruction_c *i)
   // CS.LIMIT can't change when in real/v8086 mode
   if (op1_16 > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("CALL16_Ep: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
@@ -571,7 +567,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP16_Ep(bxInstruction_c *i)
   // CS.LIMIT can't change when in real/v8086 mode
   if (op1_16 > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("JMP16_Ep: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
@@ -585,8 +581,6 @@ done:
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IRET16(bxInstruction_c *i)
 {
-  Bit16u ip, cs_raw, flags;
-
   invalidate_prefetch_q();
 
 #if BX_SUPPORT_VMX
@@ -598,35 +592,36 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IRET16(bxInstruction_c *i)
   BX_CPU_THIS_PTR show_flag |= Flag_iret;
 #endif
 
-  RSP_SPECULATIVE;
-
-  if (v8086_mode()) {
-    // IOPL check in stack_return_from_v86()
-    iret16_stack_return_from_v86(i);
-    goto done;
-  }
-
   if (protected_mode()) {
     iret_protected(i);
     goto done;
   }
 
-  ip     = pop_16();
-  cs_raw = pop_16(); // #SS has higher priority
-  flags  = pop_16();
+  RSP_SPECULATIVE;
 
-  // CS.LIMIT can't change when in real/v8086 mode
-  if(ip > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
-    BX_ERROR(("IRET16: instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+  if (v8086_mode()) {
+    // IOPL check in stack_return_from_v86()
+    iret16_stack_return_from_v86(i);
+  }
+  else {
+    Bit16u ip     = pop_16();
+    Bit16u cs_raw = pop_16(); // #SS has higher priority
+    Bit16u flags  = pop_16();
+
+    // CS.LIMIT can't change when in real/v8086 mode
+    if(ip > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
+      BX_ERROR(("IRET16: instruction pointer not within code segment limits"));
+      exception(BX_GP_EXCEPTION, 0);
+    }
+
+    load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
+    EIP = (Bit32u) ip;
+    write_flags(flags, /* change IOPL? */ 1, /* change IF? */ 1);
   }
 
-  load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_raw);
-  RIP = (Bit32u) ip;
-  write_flags(flags, /* change IOPL? */ 1, /* change IF? */ 1);
+  RSP_COMMIT;
 
 done:
-  RSP_COMMIT;
 
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_IRET,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, EIP);
