@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: descriptor.h,v 1.32 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: descriptor.h,v 1.35 2009/12/28 09:26:22 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2007-2009 Stanislav Shwartsman
@@ -183,13 +183,14 @@ typedef struct {
   bx_descriptor_t  cache;
 } bx_segment_reg_t;
 
-#if BX_CPU_LEVEL < 2
-  /* no GDTR or IDTR register in an 8086 */
-#else
 typedef struct {
   bx_address       base;   /* base address: 24bits=286,32bits=386,64bits=x86-64 */
   Bit16u           limit;  /* limit, 16bits */
 } bx_global_segment_reg_t;
-#endif
+
+void  parse_selector(Bit16u raw_selector, bx_selector_t *selector);
+Bit8u get_ar_byte(const bx_descriptor_t *d);
+void  set_ar_byte(bx_descriptor_t *d, Bit8u ar_byte);
+void  parse_descriptor(Bit32u dword1, Bit32u dword2, bx_descriptor_t *temp);
 
 #endif
