@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: exception.cc,v 1.143 2009/10/30 09:13:19 sshwarts Exp $
+// $Id: exception.cc,v 1.157 2010/04/14 17:33:19 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2010  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,6 +25,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#include "param_names.h"
 #include "iodev/iodev.h"
 
 #if BX_SUPPORT_X86_64==0
@@ -39,31 +34,9 @@
 #define RSP ESP
 #endif
 
-/* Exception classes.  These are used as indexes into the 'is_exception_OK'
- * array below, and are stored in the 'exception' array also
- */
-#define BX_ET_BENIGN       0
-#define BX_ET_CONTRIBUTORY 1
-#define BX_ET_PAGE_FAULT   2
-
-#define BX_ET_DOUBLE_FAULT 10
-
-static const bx_bool is_exception_OK[3][3] = {
-    { 1, 1, 1 }, /* 1st exception is BENIGN */
-    { 1, 0, 1 }, /* 1st exception is CONTRIBUTORY */
-    { 1, 0, 0 }  /* 1st exception is PAGE_FAULT */
-};
-
-#define BX_EXCEPTION_CLASS_TRAP  0
-#define BX_EXCEPTION_CLASS_FAULT 1
-#define BX_EXCEPTION_CLASS_ABORT 2
-
 #if BX_SUPPORT_X86_64
-void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error, Bit16u error_code)
+void BX_CPU_C::long_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_error, Bit16u error_code)
 {
-  // long mode interrupt
-  Bit64u desctmp1, desctmp2;
-
   bx_descriptor_t gate_descriptor, cs_descriptor;
   bx_selector_t cs_selector;
 
@@ -71,15 +44,15 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   // else #GP(vector number*8 + 2 + EXT)
   if ((vector*16 + 15) > BX_CPU_THIS_PTR idtr.limit) {
     BX_ERROR(("interrupt(long mode): vector must be within IDT table limits, IDT.limit = 0x%x", BX_CPU_THIS_PTR idtr.limit));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
-  desctmp1 = system_read_qword(BX_CPU_THIS_PTR idtr.base + vector*16);
-  desctmp2 = system_read_qword(BX_CPU_THIS_PTR idtr.base + vector*16 + 8);
+  Bit64u desctmp1 = system_read_qword(BX_CPU_THIS_PTR idtr.base + vector*16);
+  Bit64u desctmp2 = system_read_qword(BX_CPU_THIS_PTR idtr.base + vector*16 + 8);
 
   if (desctmp2 & BX_CONST64(0x00001F0000000000)) {
     BX_ERROR(("interrupt(long mode): IDT entry extended attributes DWORD4 TYPE != 0"));
-    exception(BX_GP_EXCEPTION, vector*8+ 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8+ 2);
   }
 
   Bit32u dword1 = GET32L(desctmp1);
@@ -91,7 +64,7 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   if ((gate_descriptor.valid==0) || gate_descriptor.segment)
   {
     BX_ERROR(("interrupt(long mode): gate descriptor is not valid sys seg"));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // descriptor AR byte must indicate interrupt gate, trap gate,
@@ -101,21 +74,21 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   {
     BX_ERROR(("interrupt(long mode): unsupported gate type %u",
         (unsigned) gate_descriptor.type));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // if software interrupt, then gate descripor DPL must be >= CPL,
   // else #GP(vector * 16 + 2 + EXT)
-  if (is_INT && (gate_descriptor.dpl < CPL))
+  if (soft_int && gate_descriptor.dpl < CPL)
   {
-    BX_ERROR(("interrupt(long mode): is_INT && gate.dpl < CPL"));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    BX_ERROR(("interrupt(long mode): soft_int && gate.dpl < CPL"));
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // Gate must be present, else #NP(vector * 16 + 2 + EXT)
   if (! IS_PRESENT(gate_descriptor)) {
     BX_ERROR(("interrupt(long mode): gate.p == 0"));
-    exception(BX_NP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_NP_EXCEPTION, vector*8 + 2);
   }
 
   Bit16u gate_dest_selector = gate_descriptor.u.gate.dest_selector;
@@ -128,7 +101,7 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   // selector must be non-null else #GP(EXT)
   if ((gate_dest_selector & 0xfffc) == 0) {
     BX_ERROR(("int_trap_gate(long mode): selector null"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   parse_selector(gate_dest_selector, &cs_selector);
@@ -144,21 +117,21 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
       IS_DATA_SEGMENT(cs_descriptor.type) ||
       cs_descriptor.dpl>CPL)
   {
-    BX_ERROR(("interrupt(long mode): not accessable or not code segment"));
-    exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+    BX_ERROR(("interrupt(long mode): not accessible or not code segment"));
+    exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
   }
 
   // check that it's a 64 bit segment
   if (! IS_LONG64_SEGMENT(cs_descriptor) || cs_descriptor.u.segment.d_b)
   {
     BX_ERROR(("interrupt(long mode): must be 64 bit segment"));
-    exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
   }
 
   // segment must be present, else #NP(selector + EXT)
   if (! IS_PRESENT(cs_descriptor)) {
     BX_ERROR(("interrupt(long mode): segment not present"));
-    exception(BX_NP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+    exception(BX_NP_EXCEPTION, cs_selector.value & 0xfffc);
   }
 
   // if code segment is non-conforming and DPL < CPL then
@@ -189,7 +162,7 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
       // #SS(selector) when changing priviledge level
       BX_ERROR(("interrupt(long mode): canonical address failure %08x%08x",
          GET32H(RSP_for_cpl_x), GET32L(RSP_for_cpl_x)));
-      exception(BX_SS_EXCEPTION, old_SS & 0xfffc, 0);
+      exception(BX_SS_EXCEPTION, old_SS & 0xfffc);
     }
 
     // push old stack long pointer onto new stack
@@ -206,18 +179,11 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
       write_new_stack_qword_64(RSP_for_cpl_x, cs_descriptor.dpl, error_code);
     }
 
-    bx_selector_t ss_selector;
-    bx_descriptor_t ss_descriptor;
-
-    // set up a null descriptor
-    parse_selector(0, &ss_selector);
-    parse_descriptor(0, 0, &ss_descriptor);
-
     // load CS:RIP (guaranteed to be in 64 bit mode)
     branch_far64(&cs_selector, &cs_descriptor, gate_dest_offset, cs_descriptor.dpl);
 
     // set up null SS descriptor
-    load_ss(&ss_selector, &ss_descriptor, cs_descriptor.dpl);
+    load_null_selector(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS], cs_descriptor.dpl);
     RSP = RSP_for_cpl_x;
 
     // if INTERRUPT GATE set IF to 0
@@ -236,37 +202,39 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   {
     BX_DEBUG(("interrupt(long mode): INTERRUPT TO SAME PRIVILEGE"));
 
-    Bit64u old_RSP = RSP;
+    Bit64u old_RSP = RSP, temp_RSP = RSP;
 
     // check selector and descriptor for new stack in current TSS
     if (ist > 0) {
       BX_DEBUG(("interrupt(long mode): trap to IST, vector = %d", ist));
-      RSP = get_RSP_from_TSS(ist+3);
+      temp_RSP = get_RSP_from_TSS(ist+3);
     }
 
     // align stack
-    RSP &= BX_CONST64(0xfffffffffffffff0);
+    temp_RSP &= BX_CONST64(0xfffffffffffffff0);
 
     // push flags onto stack
     // push current CS selector onto stack
     // push return offset onto stack
-    write_new_stack_qword_64(RSP - 8,  cs_descriptor.dpl,
+    write_new_stack_qword_64(temp_RSP - 8,  cs_descriptor.dpl,
          BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value);
-    write_new_stack_qword_64(RSP - 16, cs_descriptor.dpl, old_RSP);
-    write_new_stack_qword_64(RSP - 24, cs_descriptor.dpl, read_eflags());
-    write_new_stack_qword_64(RSP - 32, cs_descriptor.dpl,
+    write_new_stack_qword_64(temp_RSP - 16, cs_descriptor.dpl, old_RSP);
+    write_new_stack_qword_64(temp_RSP - 24, cs_descriptor.dpl, read_eflags());
+    write_new_stack_qword_64(temp_RSP - 32, cs_descriptor.dpl,
          BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value);
-    write_new_stack_qword_64(RSP - 40, cs_descriptor.dpl, RIP);
-    RSP -= 40;
+    write_new_stack_qword_64(temp_RSP - 40, cs_descriptor.dpl, RIP);
+    temp_RSP -= 40;
 
     if (push_error) {
-      RSP -= 8;
-      write_new_stack_qword_64(RSP, cs_descriptor.dpl, error_code);
+      temp_RSP -= 8;
+      write_new_stack_qword_64(temp_RSP, cs_descriptor.dpl, error_code);
     }
 
     // set the RPL field of CS to CPL
     branch_far64(&cs_selector, &cs_descriptor, gate_dest_offset, CPL);
 
+    RSP = temp_RSP;
+
     // if interrupt gate then set IF to 0
     if (!(gate_descriptor.type & 1)) // even is int-gate
       BX_CPU_THIS_PTR clear_IF();
@@ -281,14 +249,12 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   BX_ERROR(("interrupt(long mode): bad descriptor type=%u, descriptor.dpl=%u, CPL=%u",
     (unsigned) cs_descriptor.type, (unsigned) cs_descriptor.dpl, (unsigned) CPL));
   BX_ERROR(("cs.segment = %u", (unsigned) cs_descriptor.segment));
-  exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+  exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
 }
 #endif
 
-void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error, Bit16u error_code)
+void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_error, Bit16u error_code)
 {
-  // protected mode interrupt
-  Bit32u dword1, dword2;
   bx_descriptor_t gate_descriptor, cs_descriptor;
   bx_selector_t cs_selector;
 
@@ -303,19 +269,19 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
   // else #GP(vector number*8 + 2 + EXT)
   if ((vector*8 + 7) > BX_CPU_THIS_PTR idtr.limit) {
     BX_ERROR(("interrupt(): vector must be within IDT table limits, IDT.limit = 0x%x", BX_CPU_THIS_PTR idtr.limit));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   Bit64u desctmp = system_read_qword(BX_CPU_THIS_PTR idtr.base + vector*8);
 
-  dword1 = GET32L(desctmp);
-  dword2 = GET32H(desctmp);
+  Bit32u dword1 = GET32L(desctmp);
+  Bit32u dword2 = GET32H(desctmp);
 
   parse_descriptor(dword1, dword2, &gate_descriptor);
 
   if ((gate_descriptor.valid==0) || gate_descriptor.segment) {
     BX_ERROR(("interrupt(): gate descriptor is not valid sys seg (vector=0x%02x)", vector));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // descriptor AR byte must indicate interrupt gate, trap gate,
@@ -330,20 +296,20 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
   default:
     BX_ERROR(("interrupt(): gate.type(%u) != {5,6,7,14,15}",
       (unsigned) gate_descriptor.type));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // if software interrupt, then gate descripor DPL must be >= CPL,
   // else #GP(vector * 8 + 2 + EXT)
-  if (is_INT && (gate_descriptor.dpl < CPL)) {
-    BX_ERROR(("interrupt(): is_INT && (gate.dpl < CPL)"));
-    exception(BX_GP_EXCEPTION, vector*8 + 2, 0);
+  if (soft_int && gate_descriptor.dpl < CPL) {
+    BX_ERROR(("interrupt(): soft_int && (gate.dpl < CPL)"));
+    exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
   // Gate must be present, else #NP(vector * 8 + 2 + EXT)
   if (! IS_PRESENT(gate_descriptor)) {
     BX_ERROR(("interrupt(): gate not present"));
-    exception(BX_NP_EXCEPTION, vector*8 + 2, 0);
+    exception(BX_NP_EXCEPTION, vector*8 + 2);
   }
 
   switch (gate_descriptor.type) {
@@ -356,7 +322,7 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
     //      else #GP(TSS selector)
     if (tss_selector.ti) {
       BX_ERROR(("interrupt(): tss_selector.ti=1 from gate descriptor - #GP(tss_selector)"));
-      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
+      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc);
     }
 
     // index must be within GDT limits, else #TS(TSS selector)
@@ -368,26 +334,28 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
     //   else #GP(TSS selector)
     if (tss_descriptor.valid==0 || tss_descriptor.segment) {
       BX_ERROR(("interrupt(): TSS selector points to invalid or bad TSS - #GP(tss_selector)"));
-      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
+      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc);
     }
 
     if (tss_descriptor.type!=BX_SYS_SEGMENT_AVAIL_286_TSS &&
         tss_descriptor.type!=BX_SYS_SEGMENT_AVAIL_386_TSS)
     {
       BX_ERROR(("interrupt(): TSS selector points to bad TSS - #GP(tss_selector)"));
-      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
+      exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc);
     }
 
     // TSS must be present, else #NP(TSS selector)
     if (! IS_PRESENT(tss_descriptor)) {
       BX_ERROR(("interrupt(): TSS descriptor.p == 0"));
-      exception(BX_NP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
+      exception(BX_NP_EXCEPTION, raw_tss_selector & 0xfffc);
     }
 
     // switch tasks with nesting to TSS
     task_switch(0, &tss_selector, &tss_descriptor,
                     BX_TASK_FROM_INT, dword1, dword2);
 
+    RSP_SPECULATIVE;
+
     // if interrupt was caused by fault with error code
     //   stack limits must allow push of 2 more bytes, else #SS(0)
     // push error code onto stack
@@ -402,9 +370,11 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
     // instruction pointer must be in CS limit, else #GP(0)
     if (EIP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
       BX_ERROR(("interrupt(): EIP > CS.limit"));
-      exception(BX_GP_EXCEPTION, 0, 0);
+      exception(BX_GP_EXCEPTION, 0);
     }
 
+    RSP_COMMIT;
+
     return;
 
   case BX_286_INTERRUPT_GATE:
@@ -418,7 +388,7 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
     // selector must be non-null else #GP(EXT)
     if ((gate_dest_selector & 0xfffc) == 0) {
       BX_ERROR(("int_trap_gate(): selector null"));
-      exception(BX_GP_EXCEPTION, 0, 0);
+      exception(BX_GP_EXCEPTION, 0);
     }
 
     parse_selector(gate_dest_selector, &cs_selector);
@@ -434,19 +404,19 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
         IS_DATA_SEGMENT(cs_descriptor.type) ||
         cs_descriptor.dpl>CPL)
     {
-      BX_ERROR(("interrupt(): not accessable or not code segment cs=0x%04x", cs_selector.value));
-      exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+      BX_ERROR(("interrupt(): not accessible or not code segment cs=0x%04x", cs_selector.value));
+      exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
     }
 
     // segment must be present, else #NP(selector + EXT)
     if (! IS_PRESENT(cs_descriptor)) {
       BX_ERROR(("interrupt(): segment not present"));
-      exception(BX_NP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+      exception(BX_NP_EXCEPTION, cs_selector.value & 0xfffc);
     }
 
     if (cs_descriptor.dpl > CPL) {
       BX_ERROR(("interrupt(): code segment DPL(%d) > CPL", cs_descriptor.dpl));
-      exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+      exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
     }
 
     // if code segment is non-conforming and DPL < CPL then
@@ -468,13 +438,13 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
       if (is_v8086_mode && cs_descriptor.dpl != 0) {
         // if code segment DPL != 0 then #GP(new code segment selector)
         BX_ERROR(("interrupt(): code segment DPL(%d) != 0 in v8086 mode", cs_descriptor.dpl));
-        exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+        exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
       }
 
       // Selector must be non-null else #TS(EXT)
       if ((SS_for_cpl_x & 0xfffc) == 0) {
         BX_ERROR(("interrupt(): SS selector null"));
-        exception(BX_TS_EXCEPTION, 0, 0); /* TS(ext) */
+        exception(BX_TS_EXCEPTION, 0); /* TS(ext) */
       }
 
       // selector index must be within its descriptor table limits
@@ -488,14 +458,14 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
       // else #TS(SS selector + ext)
       if (ss_selector.rpl != cs_descriptor.dpl) {
         BX_ERROR(("interrupt(): SS.rpl != CS.dpl"));
-        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc, 0);
+        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc);
       }
 
       // stack seg DPL must = DPL of code segment,
       // else #TS(SS selector + ext)
       if (ss_descriptor.dpl != cs_descriptor.dpl) {
         BX_ERROR(("interrupt(): SS.dpl != CS.dpl"));
-        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc, 0);
+        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc);
       }
 
       // descriptor must indicate writable data segment,
@@ -505,19 +475,19 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
           !IS_DATA_SEGMENT_WRITEABLE(ss_descriptor.type))
       {
         BX_ERROR(("interrupt(): SS is not writable data segment"));
-        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc, 0);
+        exception(BX_TS_EXCEPTION, SS_for_cpl_x & 0xfffc);
       }
 
       // seg must be present, else #SS(SS selector + ext)
       if (! IS_PRESENT(ss_descriptor)) {
         BX_ERROR(("interrupt(): SS not present"));
-        exception(BX_SS_EXCEPTION, SS_for_cpl_x & 0xfffc, 0);
+        exception(BX_SS_EXCEPTION, SS_for_cpl_x & 0xfffc);
       }
 
       // IP must be within CS segment boundaries, else #GP(0)
       if (gate_dest_offset > cs_descriptor.u.segment.limit_scaled) {
         BX_ERROR(("interrupt(): gate EIP > CS.limit"));
-        exception(BX_GP_EXCEPTION, 0, 0);
+        exception(BX_GP_EXCEPTION, 0);
       }
 
       old_ESP = ESP;
@@ -655,13 +625,14 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
         SP = temp_SP;
       }
 
-      // load new SS:eSP values from TSS
-      load_ss(&ss_selector, &ss_descriptor, cs_descriptor.dpl);
-
       // load new CS:eIP values from gate
       // set CPL to new code segment DPL
       // set RPL of CS to CPL
       load_cs(&cs_selector, &cs_descriptor, cs_descriptor.dpl);
+
+      // load new SS:eSP values from TSS
+      load_ss(&ss_selector, &ss_descriptor, cs_descriptor.dpl);
+
       EIP = gate_dest_offset;
 
       // if INTERRUPT GATE set IF to 0
@@ -693,13 +664,13 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
       if (v8086_mode() && (IS_CODE_SEGMENT_CONFORMING(cs_descriptor.type) || cs_descriptor.dpl != 0)) {
         // if code segment DPL != 0 then #GP(new code segment selector)
         BX_ERROR(("interrupt(): code segment conforming or DPL(%d) != 0 in v8086 mode", cs_descriptor.dpl));
-        exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc, 0);
+        exception(BX_GP_EXCEPTION, cs_selector.value & 0xfffc);
       }
 
       // EIP must be in CS limit else #GP(0)
       if (gate_dest_offset > cs_descriptor.u.segment.limit_scaled) {
         BX_ERROR(("interrupt(): IP > CS descriptor limit"));
-        exception(BX_GP_EXCEPTION, 0, 0);
+        exception(BX_GP_EXCEPTION, 0);
       }
 
       // push flags onto stack
@@ -742,14 +713,11 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_er
   }
 }
 
-void BX_CPU_C::real_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error, Bit16u error_code)
+void BX_CPU_C::real_mode_int(Bit8u vector, bx_bool push_error, Bit16u error_code)
 {
-  // real mode interrupt
-  Bit16u cs_selector;
-
   if ((vector*4+3) > BX_CPU_THIS_PTR idtr.limit) {
     BX_ERROR(("interrupt(real mode) vector > idtr.limit"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   push_16((Bit16u) read_eflags());
@@ -760,9 +728,10 @@ void BX_CPU_C::real_mode_int(Bit8u vector, unsigned is_INT, bx_bool push_error,
   // CS.LIMIT can't change when in real/v8086 mode
   if (new_ip > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
     BX_ERROR(("interrupt(real mode): instruction pointer not within code segment limits"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
-  cs_selector = system_read_word(BX_CPU_THIS_PTR idtr.base + 4 * vector + 2);
+
+  Bit16u cs_selector = system_read_word(BX_CPU_THIS_PTR idtr.base + 4 * vector + 2);
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], cs_selector);
   EIP = new_ip;
 
@@ -791,11 +760,11 @@ void BX_CPU_C::interrupt(Bit8u vector, unsigned type, bx_bool push_error, Bit16u
 
   invalidate_prefetch_q();
 
-  bx_bool is_INT = 0;
+  bx_bool soft_int = 0;
   switch(type) {
     case BX_SOFTWARE_INTERRUPT:
     case BX_SOFTWARE_EXCEPTION:
-      is_INT = 1;
+      soft_int = 1;
       break;
     case BX_PRIVILEGED_SOFTWARE_INTERRUPT:
     case BX_EXTERNAL_INTERRUPT:
@@ -815,28 +784,27 @@ void BX_CPU_C::interrupt(Bit8u vector, unsigned type, bx_bool push_error, Bit16u
   BX_CPU_THIS_PTR debug_trap = 0;
   BX_CPU_THIS_PTR inhibit_mask = 0;
 
-  BX_CPU_THIS_PTR save_cs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS];
-  BX_CPU_THIS_PTR save_ss = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS];
-  BX_CPU_THIS_PTR save_eip = RIP;
-  BX_CPU_THIS_PTR save_esp = RSP;
-
 #if BX_SUPPORT_VMX
   BX_CPU_THIS_PTR in_event = 1;
 #endif
 
 #if BX_SUPPORT_X86_64
   if (long_mode()) {
-    long_mode_int(vector, is_INT, push_error, error_code);
+    long_mode_int(vector, soft_int, push_error, error_code);
   }
   else
 #endif
   {
+    RSP_SPECULATIVE;
+
     if(real_mode()) {
-       real_mode_int(vector, is_INT, push_error, error_code);
+       real_mode_int(vector, push_error, error_code);
     }
     else {
-       protected_mode_int(vector, is_INT, push_error, error_code);
+       protected_mode_int(vector, soft_int, push_error, error_code);
     }
+
+    RSP_COMMIT;
   }
 
 #if BX_X86_DEBUGGER
@@ -848,6 +816,25 @@ void BX_CPU_C::interrupt(Bit8u vector, unsigned type, bx_bool push_error, Bit16u
 #endif
 }
 
+/* Exception classes.  These are used as indexes into the 'is_exception_OK'
+ * array below, and are stored in the 'exception' array also
+ */
+#define BX_ET_BENIGN       0
+#define BX_ET_CONTRIBUTORY 1
+#define BX_ET_PAGE_FAULT   2
+
+#define BX_ET_DOUBLE_FAULT 10
+
+static const bx_bool is_exception_OK[3][3] = {
+    { 1, 1, 1 }, /* 1st exception is BENIGN */
+    { 1, 0, 1 }, /* 1st exception is CONTRIBUTORY */
+    { 1, 0, 0 }  /* 1st exception is PAGE_FAULT */
+};
+
+#define BX_EXCEPTION_CLASS_TRAP  0
+#define BX_EXCEPTION_CLASS_FAULT 1
+#define BX_EXCEPTION_CLASS_ABORT 2
+
 struct BxExceptionInfo exceptions_info[BX_CPU_HANDLED_EXCEPTIONS+1] = {
   /* DE */ { BX_ET_CONTRIBUTORY, BX_EXCEPTION_CLASS_FAULT, 0 },
   /* DB */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_FAULT, 0 },
@@ -857,9 +844,9 @@ struct BxExceptionInfo exceptions_info[BX_CPU_HANDLED_EXCEPTIONS+1] = {
   /* BR */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_FAULT, 0 },
   /* UD */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_FAULT, 0 },
   /* NM */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_FAULT, 0 },
-  /* DF */ { BX_ET_DOUBLE_FAULT, BX_EXCEPTION_CLASS_ABORT, 1 },
+  /* DF */ { BX_ET_DOUBLE_FAULT, BX_EXCEPTION_CLASS_FAULT, 1 },
              // coprocessor segment overrun (286,386 only)
-  /* -- */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_ABORT, 0 },
+  /* -- */ { BX_ET_BENIGN,       BX_EXCEPTION_CLASS_FAULT, 0 },
   /* TS */ { BX_ET_CONTRIBUTORY, BX_EXCEPTION_CLASS_FAULT, 1 },
   /* NP */ { BX_ET_CONTRIBUTORY, BX_EXCEPTION_CLASS_FAULT, 1 },
   /* SS */ { BX_ET_CONTRIBUTORY, BX_EXCEPTION_CLASS_FAULT, 1 },
@@ -876,7 +863,7 @@ struct BxExceptionInfo exceptions_info[BX_CPU_HANDLED_EXCEPTIONS+1] = {
 // vector:     0..255: vector in IDT
 // error_code: if exception generates and error, push this error code
 // trap:       override exception class to TRAP
-void BX_CPU_C::exception(unsigned vector, Bit16u error_code, unsigned unused)
+void BX_CPU_C::exception(unsigned vector, Bit16u error_code)
 {
   BX_INSTR_EXCEPTION(BX_CPU_ID, vector, error_code);
 
@@ -909,14 +896,12 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code, unsigned unused)
 #endif
 
   if (BX_CPU_THIS_PTR errorno > 0) {
-    // if not initial error, restore previous register values from
-    // previous attempt to handle exception
-    BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS] = BX_CPU_THIS_PTR save_cs;
-    BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS] = BX_CPU_THIS_PTR save_ss;
-    RIP = BX_CPU_THIS_PTR save_eip;
-    RSP = BX_CPU_THIS_PTR save_esp;
-
     if (BX_CPU_THIS_PTR errorno > 2 || BX_CPU_THIS_PTR curr_exception == BX_ET_DOUBLE_FAULT) {
+      // restore RIP/RSP to value before error occurred
+      RIP = BX_CPU_THIS_PTR prev_rip;
+      if (BX_CPU_THIS_PTR speculative_rsp)
+        RSP = BX_CPU_THIS_PTR prev_rsp;
+
       debug(BX_CPU_THIS_PTR prev_rip); // print debug information to the log
 #if BX_SUPPORT_VMX
       VMexit_TripleFault();
@@ -972,7 +957,7 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code, unsigned unused)
    */
   if (BX_CPU_THIS_PTR errorno > 0 && exception_type != BX_ET_DOUBLE_FAULT) {
     if (! is_exception_OK[BX_CPU_THIS_PTR curr_exception][exception_type]) {
-      exception(BX_DF_EXCEPTION, 0, 0);
+      exception(BX_DF_EXCEPTION, 0);
     }
   }
 
@@ -980,14 +965,11 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code, unsigned unused)
   BX_CPU_THIS_PTR errorno++;
 
   if (real_mode()) {
-    // not INT, no error code pushed
-    BX_CPU_THIS_PTR interrupt(vector, BX_HARDWARE_EXCEPTION, 0, 0);
-    BX_CPU_THIS_PTR errorno = 0; // error resolved
-    longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
-  }
-  else {
-    BX_CPU_THIS_PTR interrupt(vector, BX_HARDWARE_EXCEPTION, push_error, error_code);
-    BX_CPU_THIS_PTR errorno = 0; // error resolved
-    longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
+    push_error = 0; // not INT, no error code pushed
+    error_code = 0;
   }
+
+  interrupt(vector, BX_HARDWARE_EXCEPTION, push_error, error_code);
+  BX_CPU_THIS_PTR errorno = 0; // error resolved
+  longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
 }
