@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mult64.cc,v 1.36 2009/10/27 18:30:13 sshwarts Exp $
+// $Id: mult64.cc,v 1.41 2010/04/15 19:50:57 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -92,8 +86,7 @@ void long_imul(Bit128s *product, Bit64s op1, Bit64s op2)
 
 void long_shl(Bit128u *a)
 {
-  Bit64u c;
-  c = a->lo >> 63;
+  Bit64u c = a->lo >> 63;
   a->lo <<= 1;
   a->hi <<= 1;
   a->hi |= c;
@@ -127,7 +120,7 @@ int long_le(Bit128u *a,Bit128u *b)
   }
 }
 
-void long_div(Bit128u *quotient,Bit64u *remainder,Bit128u *dividend,Bit64u divisor)
+void long_div(Bit128u *quotient,Bit64u *remainder,const Bit128u *dividend,Bit64u divisor)
 {
   /*
   n := 0;
@@ -201,7 +194,7 @@ void long_idiv(Bit128s *quotient,Bit64s *remainder,Bit128s *dividend,Bit64s divi
   if (s1 ^ s2) {
     long_neg(quotient);
   }
-  if (s2) {
+  if (s1) {
     *remainder = -*remainder;
   }
 }
@@ -253,8 +246,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_RAXEqR(bxInstruction_c *i)
    *   RDX:RAX = sign-extend of RAX
    */
 
-  /* magic compare between RDX:RAX and sign extended RAX */
+  SET_FLAGS_OSZAPC_LOGIC_64(product_128.lo);
 
+  /* magic compare between RDX:RAX and sign extended RAX */
   if (((Bit64u)(product_128.hi) + (product_128.lo >> 63)) != 0) {
     ASSERT_FLAGS_OxxxxC();
   }
@@ -267,7 +261,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_RAXEqR(bxInstruction_c *i)
 
   Bit64u op2_64 = BX_READ_64BIT_REG(i->rm());
   if (op2_64 == 0) {
-    exception(BX_DE_EXCEPTION, 0, 0);
+    exception(BX_DE_EXCEPTION, 0);
   }
 
   op1_128.lo = RAX;
@@ -281,7 +275,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_RAXEqR(bxInstruction_c *i)
   quotient_64l = quotient_128.lo;
 
   if (quotient_128.hi != 0)
-    exception(BX_DE_EXCEPTION, 0, 0);
+    exception(BX_DE_EXCEPTION, 0);
 
   /* set EFLAGS:
    * DIV affects the following flags: O,S,Z,A,P,C are undefined
@@ -302,12 +296,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_RAXEqR(bxInstruction_c *i)
 
   /* check MIN_INT case */
   if ((op1_128.hi == (Bit64s) BX_CONST64(0x8000000000000000)) && (!op1_128.lo))
-    exception(BX_DE_EXCEPTION, 0, 0);
+    exception(BX_DE_EXCEPTION, 0);
 
   Bit64s op2_64 = BX_READ_64BIT_REG(i->rm());
 
   if (op2_64 == 0) {
-    exception(BX_DE_EXCEPTION, 0, 0);
+    exception(BX_DE_EXCEPTION, 0);
   }
 
   // quotient_128 = op1_128 / op2_64;
@@ -320,7 +314,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_RAXEqR(bxInstruction_c *i)
   if ((!(quotient_128.lo & BX_CONST64(0x8000000000000000)) && quotient_128.hi != (Bit64s) 0) ||
         (quotient_128.lo & BX_CONST64(0x8000000000000000)) && quotient_128.hi != (Bit64s) BX_CONST64(0xffffffffffffffff))
   {
-    exception(BX_DE_EXCEPTION, 0, 0);
+    exception(BX_DE_EXCEPTION, 0);
   }
 
   /* set EFLAGS:
