@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: protect_ctrl.cc,v 1.95 2009/10/18 17:11:25 sshwarts Exp $
+// $Id: protect_ctrl.cc,v 1.103 2010/04/04 19:56:55 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2010  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,9 +28,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ARPL_EwGw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16;
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_DEBUG(("ARPL: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   /* op1_16 is a register or memory reference */
@@ -78,9 +72,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LAR_GvEw(bxInstruction_c *i)
   Bit32u dword3 = 0;
 #endif
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("LAR: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (i->modC0()) {
@@ -143,13 +137,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LAR_GvEw(bxInstruction_c *i)
         }
         /* fall through */
       case BX_SYS_SEGMENT_LDT:
-#if BX_CPU_LEVEL >= 3
       case BX_SYS_SEGMENT_AVAIL_386_TSS:
       case BX_SYS_SEGMENT_BUSY_386_TSS:
       case BX_386_CALL_GATE:
-#endif
 #if BX_SUPPORT_X86_64
-        if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+        if (long64_mode() || (descriptor.type == BX_386_CALL_GATE && long_mode()) ) {
           if (!fetch_raw_descriptor2_64(&selector, &dword1, &dword2, &dword3)) {
             BX_ERROR(("LAR: failed to fetch 64-bit descriptor"));
             clear_ZF();
@@ -191,9 +183,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LSL_GvEw(bxInstruction_c *i)
   Bit32u dword3 = 0;
 #endif
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("LSL: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (i->modC0()) {
@@ -235,7 +227,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LSL_GvEw(bxInstruction_c *i)
       case BX_SYS_SEGMENT_AVAIL_386_TSS:
       case BX_SYS_SEGMENT_BUSY_386_TSS:
 #if BX_SUPPORT_X86_64
-        if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+        if (long64_mode()) {
           if (!fetch_raw_descriptor2_64(&selector, &dword1, &dword2, &dword3)) {
             BX_ERROR(("LSL: failed to fetch 64-bit descriptor"));
             clear_ZF();
@@ -251,7 +243,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LSL_GvEw(bxInstruction_c *i)
         if (dword2 & 0x00800000)
           limit32 = (limit32 << 12) | 0x00000fff;
         break;
-      default:
+      default: /* rest not accepted types to LSL */
         clear_ZF();
         return;
     }
@@ -283,11 +275,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LSL_GvEw(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SLDT_Ew(bxInstruction_c *i)
 {
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("SLDT: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_LDTR_TR_ACCESS);
+#endif
+
   Bit16u val16 = BX_CPU_THIS_PTR ldtr.selector.value;
   if (i->modC0()) {
     if (i->os32L()) {
@@ -306,11 +304,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SLDT_Ew(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::STR_Ew(bxInstruction_c *i)
 {
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("STR: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_LDTR_TR_ACCESS);
+#endif
+
   Bit16u val16 = BX_CPU_THIS_PTR tr.selector.value;
   if (i->modC0()) {
     if (i->os32L()) {
@@ -338,16 +342,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
   Bit32u dword3 = 0;
 #endif
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("LLDT: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL != 0) {
     BX_ERROR(("LLDT: The current priveledge level is not 0"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_LDTR_TR_ACCESS);
+#endif
+
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
@@ -357,8 +367,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
     raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
-  invalidate_prefetch_q();
-
   /* if selector is NULL, invalidate and done */
   if ((raw_selector & 0xfffc) == 0) {
     BX_CPU_THIS_PTR ldtr.selector.value = raw_selector;
@@ -372,7 +380,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
   // #GP(selector) if the selector operand does not point into GDT
   if (selector.ti != 0) {
     BX_ERROR(("LLDT: selector.ti != 0"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
   }
 
   /* fetch descriptor; call handles out of limits checks */
@@ -393,22 +401,24 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
          descriptor.type != BX_SYS_SEGMENT_LDT)
   {
     BX_ERROR(("LLDT: doesn't point to an LDT descriptor!"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
   }
 
   /* #NP(selector) if LDT descriptor is not present */
   if (! IS_PRESENT(descriptor)) {
     BX_ERROR(("LLDT: LDT descriptor not present!"));
-    exception(BX_NP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_NP_EXCEPTION, raw_selector & 0xfffc);
   }
 
 #if BX_SUPPORT_X86_64
-  descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
-  BX_DEBUG(("64 bit LDT base = 0x%08x%08x",
-     GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
-  if (!IsCanonical(descriptor.u.segment.base)) {
-    BX_ERROR(("LLDT: non-canonical LDT descriptor base!"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+  if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+    descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
+    BX_DEBUG(("64 bit LDT base = 0x%08x%08x",
+       GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
+    if (!IsCanonical(descriptor.u.segment.base)) {
+      BX_ERROR(("LLDT: non-canonical LDT descriptor base!"));
+      exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
+    }
   }
 #endif
 
@@ -427,16 +437,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
   Bit32u dword3 = 0;
 #endif
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("LTR: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL != 0) {
     BX_ERROR(("LTR: The current priveledge level is not 0"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_LDTR_TR_ACCESS);
+#endif
+
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
@@ -446,12 +462,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
     raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
-  invalidate_prefetch_q();
-
   /* if selector is NULL, invalidate and done */
   if ((raw_selector & BX_SELECTOR_RPL_MASK) == 0) {
     BX_ERROR(("LTR: loading with NULL selector!"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   /* parse fields in selector, then check for null selector */
@@ -459,7 +473,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
 
   if (selector.ti) {
     BX_ERROR(("LTR: selector.ti != 0"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
   }
 
   /* fetch descriptor; call handles out of limits checks */
@@ -481,29 +495,31 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
           descriptor.type!=BX_SYS_SEGMENT_AVAIL_386_TSS))
   {
     BX_ERROR(("LTR: doesn't point to an available TSS descriptor!"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
   }
 
 #if BX_SUPPORT_X86_64
   if (long_mode() && descriptor.type!=BX_SYS_SEGMENT_AVAIL_386_TSS) {
     BX_ERROR(("LTR: doesn't point to an available TSS386 descriptor in long mode!"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
   }
 #endif
 
   /* #NP(selector) if TSS descriptor is not present */
   if (! IS_PRESENT(descriptor)) {
     BX_ERROR(("LTR: TSS descriptor not present!"));
-    exception(BX_NP_EXCEPTION, raw_selector & 0xfffc, 0);
+    exception(BX_NP_EXCEPTION, raw_selector & 0xfffc);
   }
 
 #if BX_SUPPORT_X86_64
-  descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
-  BX_DEBUG(("64 bit TSS base = 0x%08x%08x",
-     GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
-  if (!IsCanonical(descriptor.u.segment.base)) {
-    BX_ERROR(("LTR: non-canonical TSS descriptor base!"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
+  if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+    descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
+    BX_DEBUG(("64 bit TSS base = 0x%08x%08x",
+       GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
+    if (!IsCanonical(descriptor.u.segment.base)) {
+      BX_ERROR(("LTR: non-canonical TSS descriptor base!"));
+      exception(BX_GP_EXCEPTION, raw_selector & 0xfffc);
+    }
   }
 #endif
 
@@ -530,9 +546,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERR_Ew(bxInstruction_c *i)
   bx_selector_t selector;
   Bit32u dword1, dword2;
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("VERR: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (i->modC0()) {
@@ -620,9 +636,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERW_Ew(bxInstruction_c *i)
   bx_selector_t selector;
   Bit32u dword1, dword2;
 
-  if (real_mode() || v8086_mode()) {
+  if (! protected_mode()) {
     BX_ERROR(("VERW: not recognized in real or virtual-8086 mode"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
   if (i->modC0()) {
@@ -688,6 +704,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SGDT_Ms(bxInstruction_c *i)
 {
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
+
   Bit16u limit_16 = BX_CPU_THIS_PTR gdtr.limit;
   Bit32u base_32  = (Bit32u) BX_CPU_THIS_PTR gdtr.base;
 
@@ -701,6 +723,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT_Ms(bxInstruction_c *i)
 {
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
+
   Bit16u limit_16 = BX_CPU_THIS_PTR idtr.limit;
   Bit32u base_32  = (Bit32u) BX_CPU_THIS_PTR idtr.base;
 
@@ -716,15 +744,19 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT_Ms(bxInstruction_c *i)
 
   if (v8086_mode()) {
     BX_ERROR(("LGDT: not recognized in virtual-8086 mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   if (!real_mode() && CPL!=0) {
     BX_ERROR(("LGDT: CPL!=0 in protected mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
-  invalidate_prefetch_q();
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
 
   Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
@@ -743,15 +775,19 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LIDT_Ms(bxInstruction_c *i)
 
   if (v8086_mode()) {
     BX_ERROR(("LIDT: not recognized in virtual-8086 mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
   if (!real_mode() && CPL!=0) {
     BX_ERROR(("LIDT: CPL!=0 in protected mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
-  invalidate_prefetch_q();
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
 
   Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
@@ -770,6 +806,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SGDT64_Ms(bxInstruction_c *i)
 {
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64);
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
+
   Bit16u limit_16 = BX_CPU_THIS_PTR gdtr.limit;
   Bit64u base_64  = BX_CPU_THIS_PTR gdtr.base;
 
@@ -783,6 +825,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT64_Ms(bxInstruction_c *i)
 {
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64);
 
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
+
   Bit16u limit_16 = BX_CPU_THIS_PTR idtr.limit;
   Bit64u base_64  = BX_CPU_THIS_PTR idtr.base;
 
@@ -798,17 +846,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT64_Ms(bxInstruction_c *i)
 
   if (CPL!=0) {
     BX_ERROR(("LGDT64_Ms: CPL != 0 in long mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
-  invalidate_prefetch_q();
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   Bit64u base_64 = read_virtual_qword_64(i->seg(), eaddr + 2);
   if (! IsCanonical(base_64)) {
     BX_ERROR(("LGDT64_Ms: loaded base64 address is not in canonical form!"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
   Bit16u limit_16 = read_virtual_word_64(i->seg(), eaddr);
 
@@ -822,17 +874,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LIDT64_Ms(bxInstruction_c *i)
 
   if (CPL != 0) {
     BX_ERROR(("LIDT64_Ms: CPL != 0 in long mode"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
 
-  invalidate_prefetch_q();
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT))
+      VMexit_Instruction(i, VMX_VMEXIT_GDTR_IDTR_ACCESS);
+#endif
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   Bit64u base_64 = read_virtual_qword_64(i->seg(), eaddr + 2);
   if (! IsCanonical(base_64)) {
     BX_ERROR(("LIDT64_Ms: loaded base64 address is not in canonical form!"));
-    exception(BX_GP_EXCEPTION, 0, 0);
+    exception(BX_GP_EXCEPTION, 0);
   }
   Bit16u limit_16 = read_virtual_word_64(i->seg(), eaddr);
 
