@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc,v 1.51 2009/10/13 15:28:16 sshwarts Exp $
+// $Id: shift16.cc,v 1.53 2010/02/26 23:09:30 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -159,7 +153,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
+  of = ((Bit16u)((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -200,7 +194,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
+  of = ((Bit16u)((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -388,7 +382,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_Ew(bxInstruction_c *i)
   }
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result15 ^ result14
+  of = ((Bit16u)((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result15 ^ result14
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -476,7 +470,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_Ew(bxInstruction_c *i)
   cf = (op1_16 >> (count - 1)) & 0x1;
   // note, that of == result15 if count == 1 and
   //            of == 0        if count >= 2
-  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1;
+  of = ((Bit16u)((result_16 << 1) ^ result_16) >> 15) & 0x1;
 
   SET_FLAGS_OSZAPC_LOGIC_16(result_16);
   SET_FLAGS_OxxxxC(of, cf);
