@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: smm.cc,v 1.62 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: smm.cc,v 1.71 2010/04/13 17:56:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2006-2009 Stanislav Shwartsman
@@ -69,13 +69,19 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSM(bxInstruction_c *i)
   /* If we are not in System Management Mode, then #UD should be generated */
   if (! BX_CPU_THIS_PTR smm_mode()) {
     BX_INFO(("RSM not in System Management Mode !"));
-    exception(BX_UD_EXCEPTION, 0, 0);
+    exception(BX_UD_EXCEPTION, 0);
   }
 
 #if BX_SUPPORT_VMX
-  if (BX_CPU_THIS_PTR in_vmx_guest) {
-    BX_ERROR(("VMEXIT: RSM in VMX non-root operation"));
-    VMexit(i, VMX_VMEXIT_RSM, 0);
+  if (BX_CPU_THIS_PTR in_vmx) {
+    if (BX_CPU_THIS_PTR in_vmx_guest) {
+      BX_ERROR(("VMEXIT: RSM in VMX non-root operation"));
+      VMexit(i, VMX_VMEXIT_RSM, 0);
+    }
+    else {
+      BX_ERROR(("RSM in VMX root operation !"));
+      exception(BX_UD_EXCEPTION, 0);
+    }
   }
 #endif
 
@@ -94,7 +100,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSM(bxInstruction_c *i)
   for(n=0;n<SMM_SAVE_STATE_MAP_SIZE;n++) {
     base -= 4;
     access_read_physical(base, 4, &saved_state[n]);
-    BX_DBG_PHY_MEMORY_ACCESS(BX_CPU_ID, base, 4, BX_READ, (Bit8u*)(&saved_state[n]));
+    BX_DBG_PHY_MEMORY_ACCESS(BX_CPU_ID, base, 4, BX_SMRAM_ACCESS | BX_READ, (Bit8u*)(&saved_state[n]));
   }
   BX_CPU_THIS_PTR in_smm = 0;
 
@@ -115,6 +121,36 @@ void BX_CPU_C::enter_system_management_mode(void)
 
   // debug(BX_CPU_THIS_PTR prev_rip);
 
+  //
+  // Processors that support VMX operation perform SMI delivery as follows:
+  //
+
+#if BX_SUPPORT_VMX
+  // Enter SMM
+  // save the following internal to the processor:
+  //   * CR4.VMXE
+  //   * an indication of whether the logical processor was in VMX operation (root or non-root)
+  // IF the logical processor is in VMX operation
+  // THEN
+  //   leave VMX operation;
+  //   save VMX-critical state defined below;
+  //   preserve current VMCS pointer as noted below;
+  // FI;
+  // CR4.VMXE = 0;
+
+  BX_CPU_THIS_PTR cr4.set_VMXE(0);
+  BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_vmx;
+  BX_CPU_THIS_PTR in_smm_vmx_guest = BX_CPU_THIS_PTR in_vmx_guest;
+  BX_CPU_THIS_PTR in_vmx = 0;
+  BX_CPU_THIS_PTR in_vmx_guest = 0;
+
+  BX_INFO(("enter_system_management_mode: temporary disable VMX while in SMM mode"));
+
+  // perform ordinary SMI delivery:
+  //   * save processor state in SMRAM;
+  //   * set processor state to standard SMM values
+#endif
+
   BX_CPU_THIS_PTR in_smm = 1;
   BX_CPU_THIS_PTR disable_NMI = 1;
 
@@ -122,14 +158,14 @@ void BX_CPU_C::enter_system_management_mode(void)
   // reset reserved bits
   for(n=0;n<SMM_SAVE_STATE_MAP_SIZE;n++) saved_state[n] = 0;
   // prepare CPU state to be saved in the SMRAM
-  BX_CPU_THIS_PTR smram_save_state(saved_state);
+  smram_save_state(saved_state);
 
   bx_phy_address base = BX_CPU_THIS_PTR smbase + 0x10000;
   // could be optimized with reading of only non-reserved bytes
   for(n=0;n<SMM_SAVE_STATE_MAP_SIZE;n++) {
     base -= 4;
     access_write_physical(base, 4, &saved_state[n]);
-    BX_DBG_PHY_MEMORY_ACCESS(BX_CPU_ID, base, 4, BX_WRITE, (Bit8u*)(&saved_state[n]));
+    BX_DBG_PHY_MEMORY_ACCESS(BX_CPU_ID, base, 4, BX_SMRAM_ACCESS | BX_WRITE, (Bit8u*)(&saved_state[n]));
   }
 
   BX_CPU_THIS_PTR setEFlags(0x2); // Bit1 is always set
@@ -142,13 +178,13 @@ void BX_CPU_C::enter_system_management_mode(void)
   BX_CPU_THIS_PTR cr0.set_TS(0); // no task switch (bit 3)
   BX_CPU_THIS_PTR cr0.set_PG(0); // paging disabled (bit 31)
 
-  // paging mode was changed - flush TLB
-  TLB_flush(); //  Flush Global entries also
-
 #if BX_CPU_LEVEL >= 4
   BX_CPU_THIS_PTR cr4.set32(0);
 #endif
 
+  // paging mode was changed - flush TLB
+  TLB_flush(); //  Flush Global entries also
+
 #if BX_SUPPORT_X86_64
   BX_CPU_THIS_PTR efer.set32(0);
 #endif
@@ -464,25 +500,64 @@ bx_bool BX_CPU_C::smram_restore_state(const Bit32u *saved_state)
   Bit32u temp_cr0    = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR0);
   Bit32u temp_eflags = SMRAM_FIELD(saved_state, SMRAM_FIELD_EFLAGS);
   Bit32u temp_efer   = SMRAM_FIELD(saved_state, SMRAM_FIELD_EFER);
+  Bit32u temp_cr4    = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR4);
 
-  bx_bool pe = (temp_cr0 & 0x1);
-  bx_bool nw = (temp_cr0 >> 29) & 0x1;
-  bx_bool cd = (temp_cr0 >> 30) & 0x1;
-  bx_bool pg = (temp_cr0 >> 31) & 0x1;
+  // Processors that support VMX operation perform RSM as follows:
+#if BX_SUPPORT_VMX
+  // IF VMXE=1 in CR4 image in SMRAM
+  // THEN
+  //   fail and enter shutdown state;
 
-  // check CR0 conditions for entering to shutdown state
-  if (pg && !pe) {
-    BX_PANIC(("SMM restore: attempt to set CR0.PG with CR0.PE cleared !"));
+  if (temp_cr4 & (1 << 13)) {
+    BX_PANIC(("SMM restore: CR4.VMXE is set in restore image !"));
     return 0;
   }
 
-  if (nw && !cd) {
-    BX_PANIC(("SMM restore: attempt to set CR0.NW with CR0.CD cleared !"));
+  // restore state normally from SMRAM;
+  // CR4.VMXE = value stored internally;
+  // IF internal storage indicates that the logical processor had been in VMX operation (root or non-root)
+  // THEN
+  //   enter VMX operation (root or non-root);
+  //   restore VMX-critical state
+  //   set CR0.PE, CR0.NE, and CR0.PG to 1;
+  //   IF RFLAGS.VM = 0
+  //   THEN
+  //     CS.RPL = SS.DPL;
+  //     SS.RPL = SS.DPL;
+  //   FI;
+  //   If necessary, restore current VMCS pointer;
+  //   Leave SMM; Deassert SMMEM on subsequent bus transactions;
+  //   IF logical processor will be in VMX operation after RSM
+  //   THEN
+  //     block A20M and leave A20M mode;
+  //   FI;
+
+  if (BX_CPU_THIS_PTR in_smm_vmx) {
+    BX_CPU_THIS_PTR in_vmx = 1;
+    BX_CPU_THIS_PTR in_vmx_guest = BX_CPU_THIS_PTR in_smm_vmx_guest;
+    BX_INFO(("SMM Restore: enable VMX %s mode", BX_CPU_THIS_PTR in_vmx_guest ? "guest" : "host"));
+    temp_cr4 |= (1<<13); /* set VMXE */
+    temp_cr0 |= (1<<31)  /* PG */ | (1 << 5) /* NE */ | 0x1 /* PE */;
+    // block and disable A20M;
+  }
+#endif
+
+  bx_bool pe = (temp_cr0 & 0x1);
+  bx_bool pg = (temp_cr0 >> 31) & 0x1;
+
+  // check CR0 conditions for entering to shutdown state
+  if (!check_CR0(temp_cr0)) {
+    BX_PANIC(("SMM restore: CR0 consistency check failed !"));
+    return 0;
+  }
+
+  if (!check_CR4(temp_cr4)) {
+    BX_PANIC(("SMM restore: CR4 consistency check failed !"));
     return 0;
   }
 
   // shutdown if write to reserved CR4 bits
-  if (! SetCR4(SMRAM_FIELD(saved_state, SMRAM_FIELD_CR4))) {
+  if (!SetCR4(temp_cr4)) {
     BX_PANIC(("SMM restore: incorrect CR4 state !"));
     return 0;
   }
@@ -523,7 +598,17 @@ bx_bool BX_CPU_C::smram_restore_state(const Bit32u *saved_state)
   setEFlags(temp_eflags);
 
   bx_phy_address temp_cr3 = (bx_phy_address) SMRAM_FIELD64(saved_state, SMRAM_FIELD_CR3_HI32, SMRAM_FIELD_CR3);
-  SetCR3(temp_cr3);
+  if (!SetCR3(temp_cr3)) {
+    BX_PANIC(("SMM restore: failed to restore CR3 !"));
+    return 0;
+  }
+
+  if (BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr4.get_PAE() && !long_mode()) {
+    if (! CheckPDPTR(temp_cr3)) {
+      BX_ERROR(("SMM restore: PDPTR check failed !"));
+      return 0;
+    }
+  }
 
   for (int n=0; n<BX_GENERAL_REGISTERS; n++) {
     Bit64u val_64 = SMRAM_FIELD64(saved_state,
@@ -652,40 +737,50 @@ void BX_CPU_C::smram_save_state(Bit32u *saved_state)
 
 bx_bool BX_CPU_C::smram_restore_state(const Bit32u *saved_state)
 {
-  Bit32u temp_cr0    = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR0);
-  Bit32u temp_eflags = SMRAM_FIELD(saved_state, SMRAM_FIELD_EFLAGS);
-  Bit32u temp_cr3    = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR3);
-
-  bx_bool pe = (temp_cr0 & 0x01);
-  bx_bool nw = (temp_cr0 >> 29) & 0x01;
-  bx_bool cd = (temp_cr0 >> 30) & 0x01;
-  bx_bool pg = (temp_cr0 >> 31) & 0x01;
-
   // check conditions for entering to shutdown state
-  if (pg && !pe) {
-    BX_PANIC(("SMM restore: attempt to set CR0.PG with CR0.PE cleared !"));
+  Bit32u temp_cr0 = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR0);
+  if (!check_CR0(temp_cr0)) {
+    BX_PANIC(("SMM restore: CR0 consistency check failed !"));
     return 0;
   }
 
-  if (nw && !cd) {
-    BX_PANIC(("SMM restore: attempt to set CR0.NW with CR0.CD cleared !"));
+#if BX_CPU_LEVEL >= 4
+  Bit32u temp_cr4 = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR4);
+  if (! check_CR4(temp_cr4)) {
+    BX_PANIC(("SMM restore: CR4 consistency check failed !"));
     return 0;
   }
+#endif
 
   if (!SetCR0(temp_cr0)) {
     BX_PANIC(("SMM restore: failed to restore CR0 !"));
     return 0;
   }
-  SetCR3(temp_cr3);
-  setEFlags(temp_eflags);
-
 #if BX_CPU_LEVEL >= 4
-  if (! SetCR4(SMRAM_FIELD(saved_state, SMRAM_FIELD_CR4))) {
+  if (!SetCR4(temp_cr4)) {
     BX_PANIC(("SMM restore: incorrect CR4 state !"));
     return 0;
   }
 #endif
 
+  Bit32u temp_cr3 = SMRAM_FIELD(saved_state, SMRAM_FIELD_CR3);
+  if (!SetCR3(temp_cr3)) {
+    BX_PANIC(("SMM restore: failed to restore CR3 !"));
+    return 0;
+  }
+
+#if BX_CPU_LEVEL >= 6
+  if (BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr4.get_PAE()) {
+    if (! CheckPDPTR(temp_cr3)) {
+      BX_ERROR(("SMM restore: PDPTR check failed !"));
+      return 0;
+    }
+  }
+#endif
+
+  Bit32u temp_eflags = SMRAM_FIELD(saved_state, SMRAM_FIELD_EFLAGS);
+  setEFlags(temp_eflags);
+
   for (int n=0; n<BX_GENERAL_REGISTERS; n++) {
     Bit32u val_32 = SMRAM_FIELD(saved_state, SMRAM_FIELD_EAX + n);
     BX_WRITE_32BIT_REGZ(n, val_32);
