@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: soft_int.cc,v 1.55 2009/11/02 15:00:47 sshwarts Exp $
+// $Id: soft_int.cc,v 1.61 2010/03/24 21:26:13 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -46,7 +40,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BOUND_GwMa(bxInstruction_c *i)
 
   if (op1_16 < bound_min || op1_16 > bound_max) {
     BX_INFO(("BOUND_GdMa: fails bounds test"));
-    exception(BX_BR_EXCEPTION, 0, 0);
+    exception(BX_BR_EXCEPTION, 0);
   }
 }
 
@@ -61,29 +55,28 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BOUND_GdMa(bxInstruction_c *i)
 
   if (op1_32 < bound_min || op1_32 > bound_max) {
     BX_INFO(("BOUND_GdMa: fails bounds test"));
-    exception(BX_BR_EXCEPTION, 0, 0);
+    exception(BX_BR_EXCEPTION, 0);
   }
 }
 
+// This is an undocumented instrucion (opcode 0xf1) which
+// is useful for an ICE system
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INT1(bxInstruction_c *i)
 {
 #if BX_SUPPORT_VMX
   VMexit_Event(i, BX_PRIVILEGED_SOFTWARE_INTERRUPT, 1, 0, 0);
 #endif
 
-  // This is an undocumented instrucion (opcode 0xf1)
-  // which is useful for an ICE system.
-
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR show_flag |= Flag_softint;
 #endif
 
-  RSP_SPECULATIVE;
+  BX_CPU_THIS_PTR EXT = 1;
 
   // interrupt is not RSP safe
   interrupt(1, BX_PRIVILEGED_SOFTWARE_INTERRUPT, 0, 0);
 
-  RSP_COMMIT;
+  BX_CPU_THIS_PTR EXT = 0;
 
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_INT,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value,
@@ -102,13 +95,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INT3(bxInstruction_c *i)
   BX_CPU_THIS_PTR show_flag |= Flag_softint;
 #endif
 
-  RSP_SPECULATIVE;
-
   // interrupt is not RSP safe
   interrupt(3, BX_SOFTWARE_EXCEPTION, 0, 0);
 
-  RSP_COMMIT;
-
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_INT,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value,
                       EIP);
@@ -123,41 +112,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INT_Ib(bxInstruction_c *i)
 
   Bit8u vector = i->Ib();
 
-  RSP_SPECULATIVE;
-
-  if (v8086_mode()) {
-#if BX_CPU_LEVEL >= 5
-    if (BX_CPU_THIS_PTR cr4.get_VME())
-    {
-      bx_address tr_base = BX_CPU_THIS_PTR tr.cache.u.segment.base;
-      if (BX_CPU_THIS_PTR tr.cache.u.segment.limit_scaled < 103) {
-        BX_ERROR(("INT_Ib(): TR.limit < 103 in VME"));
-        exception(BX_GP_EXCEPTION, 0, 0);
-      }
-
-      Bit32u io_base = system_read_word(tr_base + 102), offset = io_base - 32 + (vector >> 3);
-      if (offset > BX_CPU_THIS_PTR tr.cache.u.segment.limit_scaled) {
-        BX_ERROR(("INT_Ib(): failed to fetch VME redirection bitmap"));
-        exception(BX_GP_EXCEPTION, 0, 0);
-      }
-
-      Bit8u vme_redirection_bitmap = system_read_byte(tr_base + offset);
-      if (!(vme_redirection_bitmap & (1 << (vector & 7))))
-      {
-        // redirect interrupt through virtual-mode idt
-        v86_redirect_interrupt(vector);
-        goto done;
-      }
-    }
-#endif
-    // interrupt is not redirected or VME is OFF
-    if (BX_CPU_THIS_PTR get_IOPL() < 3)
-    {
-      BX_DEBUG(("INT_Ib(): Interrupt cannot be redirected, generate #GP(0)"));
-      exception(BX_GP_EXCEPTION, 0, 0);
-    }
-  }
-
 #if BX_SUPPORT_VMX
   VMexit_Event(i, BX_SOFTWARE_INTERRUPT, vector, 0, 0);
 #endif
@@ -168,6 +122,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INT_Ib(bxInstruction_c *i)
   }
 #endif
 
+  RSP_SPECULATIVE;
+
+  if (v8086_mode()) {
+    // redirect interrupt through virtual-mode idt
+    if (v86_redirect_interrupt(vector)) goto done;
+  }
+
   interrupt(vector, BX_SOFTWARE_INTERRUPT, 0, 0);
 
 done:
@@ -191,13 +152,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INTO(bxInstruction_c *i)
     BX_CPU_THIS_PTR show_flag |= Flag_softint;
 #endif
 
-    RSP_SPECULATIVE;
-
     // interrupt is not RSP safe
     interrupt(4, BX_SOFTWARE_EXCEPTION, 0, 0);
 
-    RSP_COMMIT;
-
     BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_INT,
                         BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value,
                         EIP);
