@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vmx.h,v 1.7 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: vmx.h,v 1.30 2010/04/24 09:36:04 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2009 Stanislav Shwartsman
@@ -30,6 +30,13 @@
 // VMCS pointer is always 64-bit variable
 #define BX_INVALID_VMCSPTR BX_CONST64(0xFFFFFFFFFFFFFFFF)
 
+// bits supported in IA32_FEATURE_CONTROL MSR
+#define BX_IA32_FEATURE_CONTROL_LOCK_BIT       0x1
+#define BX_IA32_FEATURE_CONTROL_VMX_ENABLE_BIT 0x4
+
+#define BX_IA32_FEATURE_CONTROL_BITS \
+   (BX_IA32_FEATURE_CONTROL_LOCK_BIT | BX_IA32_FEATURE_CONTROL_VMX_ENABLE_BIT)
+
 // VMX error codes
 enum VMX_error_code {
     VMXERR_NO_ERROR = 0,
@@ -59,7 +66,8 @@ enum VMX_error_code {
     VMXERR_VMCALL_WITH_INVALID_SMM_MONITOR_FEATURES = 24,
     VMXERR_VMENTRY_INVALID_VM_CONTROL_FIELD_IN_EXECUTIVE_VMCS = 25,
     VMXERR_VMENTRY_MOV_SS_BLOCKING = 26,
-    VMXERR_RESERVED27 = 27
+    VMXERR_RESERVED27 = 27,
+    VMXERR_INVALID_INVEPT_INVVPID = 28
 };
 
 enum VMX_vmexit_reason {
@@ -146,10 +154,20 @@ enum VMX_vmabort_code {
    VMABORT_VMEXIT_MACHINE_CHECK_ERROR
 };
 
+// VMX APIC ACCESS VMEXIT qualification
+#define VMX_APIC_READ_INSTRUCTION_EXECUTION   0x0000
+#define VMX_APIC_WRITE_INSTRUCTION_EXECUTION  0x1000
+#define VMX_APIC_INSTRUCTION_FETCH            0x2000 /* won't happen because cpu::prefetch will crash */
+#define VMX_APIC_ACCESS_DURING_EVENT_DELIVERY 0x3000
+
 // =============
 //  VMCS fields
 // =============
 
+/* VMCS 16-bit control fields */
+/* binary 0000_00xx_xxxx_xxx0 */
+#define VMCS_16BIT_CONTROL_VPID                            0x00000000
+
 /* VMCS 16-bit guest-state fields */
 /* binary 0000_10xx_xxxx_xxx0 */
 #define VMCS_16BIT_GUEST_ES_SELECTOR                       0x00000800
@@ -193,6 +211,13 @@ enum VMX_vmabort_code {
 #define VMCS_64BIT_CONTROL_VIRTUAL_APIC_PAGE_ADDR_HI       0x00002013
 #define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR                0x00002014
 #define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR_HI             0x00002015
+#define VMCS_64BIT_CONTROL_EPTPTR                          0x0000201A
+#define VMCS_64BIT_CONTROL_EPTPTR_HI                       0x0000201B
+
+/* VMCS 64-bit read only data fields */
+/* binary 0010_01xx_xxxx_xxx0 */
+#define VMCS_64BIT_GUEST_PHYSICAL_ADDR                     0x00002400
+#define VMCS_64BIT_GUEST_PHYSICAL_ADDR_HI                  0x00002401
 
 /* VMCS 64-bit guest state fields */
 /* binary 0010_10xx_xxxx_xxx0 */
@@ -204,6 +229,16 @@ enum VMX_vmabort_code {
 #define VMCS_64BIT_GUEST_IA32_PAT_HI                       0x00002805
 #define VMCS_64BIT_GUEST_IA32_EFER                         0x00002806
 #define VMCS_64BIT_GUEST_IA32_EFER_HI                      0x00002807
+#define VMCS_64BIT_GUEST_IA32_PERF_GLOBAL_CTRL             0x00002808
+#define VMCS_64BIT_GUEST_IA32_PERF_GLOBAL_CTRL_HI          0x00002809
+#define VMCS_64BIT_GUEST_IA32_PDPTE0                       0x0000280A
+#define VMCS_64BIT_GUEST_IA32_PDPTE0_HI                    0x0000280B
+#define VMCS_64BIT_GUEST_IA32_PDPTE1                       0x0000280C
+#define VMCS_64BIT_GUEST_IA32_PDPTE1_HI                    0x0000280D
+#define VMCS_64BIT_GUEST_IA32_PDPTE2                       0x0000280E
+#define VMCS_64BIT_GUEST_IA32_PDPTE2_HI                    0x0000280F
+#define VMCS_64BIT_GUEST_IA32_PDPTE3                       0x00002810
+#define VMCS_64BIT_GUEST_IA32_PDPTE3_HI                    0x00002811
 
 /* VMCS 64-bit host state fields */
 /* binary 0010_11xx_xxxx_xxx0 */
@@ -211,11 +246,13 @@ enum VMX_vmabort_code {
 #define VMCS_64BIT_HOST_IA32_PAT_HI                        0x00002C01
 #define VMCS_64BIT_HOST_IA32_EFER                          0x00002C02
 #define VMCS_64BIT_HOST_IA32_EFER_HI                       0x00002C03
+#define VMCS_64BIT_HOST_IA32_PERF_GLOBAL_CTRL              0x00002C04
+#define VMCS_64BIT_HOST_IA32_PERF_GLOBAL_CTRL_HI           0x00002C05
 
 /* VMCS 32_bit control fields */
 /* binary 0100_00xx_xxxx_xxx0 */
 #define VMCS_32BIT_CONTROL_PIN_BASED_EXEC_CONTROLS         0x00004000
-#define VMCS_32BIT_CONTROL_PROCESSOR_BASED_EXEC_CONTROLS   0x00004002
+#define VMCS_32BIT_CONTROL_PROCESSOR_BASED_VMEXEC_CONTROLS 0x00004002
 #define VMCS_32BIT_CONTROL_EXECUTION_BITMAP                0x00004004
 #define VMCS_32BIT_CONTROL_PAGE_FAULT_ERR_CODE_MASK        0x00004006
 #define VMCS_32BIT_CONTROL_PAGE_FAULT_ERR_CODE_MATCH       0x00004008
@@ -229,6 +266,7 @@ enum VMX_vmabort_code {
 #define VMCS_32BIT_CONTROL_VMENTRY_EXCEPTION_ERR_CODE      0x00004018
 #define VMCS_32BIT_CONTROL_VMENTRY_INSTRUCTION_LENGTH      0x0000401A
 #define VMCS_32BIT_CONTROL_TPR_THRESHOLD                   0x0000401C
+#define VMCS_32BIT_CONTROL_SECONDARY_VMEXEC_CONTROLS       0x0000401E
 
 /* VMCS 32-bit read only data fields */
 /* binary 0100_01xx_xxxx_xxx0 */
@@ -415,9 +453,12 @@ typedef struct bx_VMCS_GUEST_STATE
 
    Bit64u link_pointer;
 
-   Bit64u pat_msr;
+#if BX_SUPPORT_VMX >= 2
 #if BX_SUPPORT_X86_64
    Bit64u efer_msr;
+#endif
+   Bit64u pat_msr;
+   Bit64u pdptr[4];
 #endif
 } VMCS_GUEST_STATE;
 
@@ -445,9 +486,11 @@ typedef struct bx_VMCS_HOST_STATE
    bx_address sysenter_eip_msr;
    Bit32u sysenter_cs_msr;
 
-   Bit64u pat_msr;
+#if BX_SUPPORT_VMX >= 2
 #if BX_SUPPORT_X86_64
    Bit64u efer_msr;
+#endif
+   Bit64u pat_msr;
 #endif
 } VMCS_HOST_STATE;
 
@@ -469,8 +512,8 @@ typedef struct bx_VMCS
 #else // only really supported features
 
 #define VMX_VM_EXEC_CTRL1_SUPPORTED_BITS \
-       (VMX_VM_EXEC_CTRL1_EXTERNAL_INTERRUPT_VMEXIT | \
-        VMX_VM_EXEC_CTRL1_NMI_VMEXIT)
+  (VMX_VM_EXEC_CTRL1_EXTERNAL_INTERRUPT_VMEXIT | \
+   VMX_VM_EXEC_CTRL1_NMI_VMEXIT)
 
 #endif
 
@@ -505,28 +548,61 @@ typedef struct bx_VMCS
 #else // only really supported features
 
 #define VMX_VM_EXEC_CTRL2_SUPPORTED_BITS \
-       (VMX_VM_EXEC_CTRL2_INTERRUPT_WINDOW_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_TSC_OFFSET | \
-        VMX_VM_EXEC_CTRL2_HLT_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_INVLPG_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_MWAIT_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_RDPMC_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_CR3_WRITE_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_CR3_READ_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_CR8_WRITE_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_CR8_READ_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_NMI_WINDOW_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_DRx_ACCESS_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_IO_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_IO_BITMAPS | \
-        VMX_VM_EXEC_CTRL2_MSR_BITMAPS | \
-        VMX_VM_EXEC_CTRL2_MONITOR_VMEXIT | \
-        VMX_VM_EXEC_CTRL2_PAUSE_VMEXIT)
+   (VMX_VM_EXEC_CTRL2_INTERRUPT_WINDOW_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_TSC_OFFSET | \
+    VMX_VM_EXEC_CTRL2_HLT_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_INVLPG_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_MWAIT_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_RDPMC_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_CR3_WRITE_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_CR3_READ_VMEXIT | \
+   (BX_SUPPORT_X86_64 ? VMX_VM_EXEC_CTRL2_CR8_WRITE_VMEXIT : 0) | \
+   (BX_SUPPORT_X86_64 ? VMX_VM_EXEC_CTRL2_CR8_READ_VMEXIT : 0) | \
+   (BX_SUPPORT_X86_64 ? VMX_VM_EXEC_CTRL2_TPR_SHADOW : 0) | \
+    VMX_VM_EXEC_CTRL2_NMI_WINDOW_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_DRx_ACCESS_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_IO_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_IO_BITMAPS | \
+    VMX_VM_EXEC_CTRL2_MSR_BITMAPS | \
+    VMX_VM_EXEC_CTRL2_MONITOR_VMEXIT | \
+    VMX_VM_EXEC_CTRL2_PAUSE_VMEXIT | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VM_EXEC_CTRL2_SECONDARY_CONTROLS : 0))
 
 #endif
 
    Bit32u vmexec_ctrls2;
+
+#define VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES  (1 << 0)
+#define VMX_VM_EXEC_CTRL3_EPT_ENABLE                (1 << 1)
+#define VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT   (1 << 2)
+#define VMX_VM_EXEC_CTRL3_RDTSCP                    (1 << 3)
+#define VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE    (1 << 4)
+#define VMX_VM_EXEC_CTRL3_VPID_ENABLE               (1 << 5)
+#define VMX_VM_EXEC_CTRL3_WBINVD_VMEXIT             (1 << 6)
+#define VMX_VM_EXEC_CTRL3_UNRESTRICTED_GUEST        (1 << 7)
+#define VMX_VM_EXEC_CTRL3_PAUSE_LOOP_VMEXIT         (1 << 10)
+
+#ifdef BX_VMX_ENABLE_ALL
+
+#define VMX_VM_EXEC_CTRL3_SUPPORTED_BITS (0x000004ff)
+
+#else // only really supported features
+
+#define VMX_VM_EXEC_CTRL3_SUPPORTED_BITS \
+   (VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES | \
+    VMX_VM_EXEC_CTRL3_EPT_ENABLE | \
+    VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT | \
+    VMX_VM_EXEC_CTRL3_RDTSCP | \
+    VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE | \
+    VMX_VM_EXEC_CTRL3_VPID_ENABLE | \
+    VMX_VM_EXEC_CTRL3_WBINVD_VMEXIT | \
+    VMX_VM_EXEC_CTRL3_UNRESTRICTED_GUEST)
+
+#endif
+
+   Bit32u vmexec_ctrls3;
+
    Bit32u vm_exceptions_bitmap;
    Bit32u vm_pf_mask;
    Bit32u vm_pf_match;
@@ -542,10 +618,15 @@ typedef struct bx_VMCS
 #define VMX_CR3_TARGET_MAX_CNT 4
 
    Bit32u vm_cr3_target_cnt;
-   bx_address vm_cr3_target_value[4];
+   bx_address vm_cr3_target_value[VMX_CR3_TARGET_MAX_CNT];
 
    bx_phy_address virtual_apic_page_addr;
    Bit32u vm_tpr_threshold;
+#if BX_SUPPORT_VMX >= 2
+   bx_phy_address apic_access_page;
+   Bit64u eptptr;
+   Bit16u vpid;
+#endif
 
    Bit64u executive_vmcsptr;
 
@@ -570,9 +651,13 @@ typedef struct bx_VMCS
 #else // only really supported features
 
 #define VMX_VMEXIT_CTRL1_SUPPORTED_BITS \
-       (VMX_VMEXIT_CTRL1_SAVE_DBG_CTRLS | \
-        VMX_VMEXIT_CTRL1_HOST_ADDR_SPACE_SIZE | \
-        VMX_VMEXIT_CTRL1_INTA_ON_VMEXIT)
+   (VMX_VMEXIT_CTRL1_SAVE_DBG_CTRLS | \
+   (BX_SUPPORT_X86_64 ? VMX_VMEXIT_CTRL1_HOST_ADDR_SPACE_SIZE : 0) | \
+    VMX_VMEXIT_CTRL1_INTA_ON_VMEXIT | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMEXIT_CTRL1_STORE_PAT_MSR : 0) | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMEXIT_CTRL1_LOAD_PAT_MSR : 0) | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMEXIT_CTRL1_STORE_EFER_MSR : 0) | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMEXIT_CTRL1_LOAD_EFER_MSR : 0))
 
 #endif
 
@@ -602,10 +687,12 @@ typedef struct bx_VMCS
 #else // only really supported features
 
 #define VMX_VMENTRY_CTRL1_SUPPORTED_BITS \
-       (VMX_VMENTRY_CTRL1_LOAD_DBG_CTRLS | \
-        VMX_VMENTRY_CTRL1_X86_64_GUEST | \
-        VMX_VMENTRY_CTRL1_SMM_ENTER | \
-        VMX_VMENTRY_CTRL1_DEACTIVATE_DUAL_MONITOR_TREATMENT)
+   (VMX_VMENTRY_CTRL1_LOAD_DBG_CTRLS | \
+   (BX_SUPPORT_X86_64 ? VMX_VMENTRY_CTRL1_X86_64_GUEST : 0) | \
+    VMX_VMENTRY_CTRL1_SMM_ENTER | \
+    VMX_VMENTRY_CTRL1_DEACTIVATE_DUAL_MONITOR_TREATMENT | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMENTRY_CTRL1_LOAD_PAT_MSR : 0) | \
+   ((BX_SUPPORT_VMX >= 2) ? VMX_VMENTRY_CTRL1_LOAD_EFER_MSR : 0))
 
 #endif
    
@@ -627,6 +714,9 @@ typedef struct bx_VMCS
    Bit32u vmexit_instr_info;
    Bit32u vmexit_instr_length;
    bx_address vmexit_guest_laddr;
+#if BX_SUPPORT_VMX >= 2
+   bx_phy_address vmexit_guest_paddr;
+#endif
    Bit32u vmexit_excep_info;
    Bit32u vmexit_excep_error_code;
 
@@ -659,6 +749,8 @@ enum VMX_Activity_State {
 #define PIN_VMEXIT(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1 & (ctrl))
 #define     VMEXIT(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls2 & (ctrl))
 
+#define SECONDARY_VMEXEC_CONTROL(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls3 & (ctrl))
+
 #define BX_VMX_INTERRUPTS_BLOCKED_BY_STI      (1 << 0)
 #define BX_VMX_INTERRUPTS_BLOCKED_BY_MOV_SS   (1 << 1)
 #define BX_VMX_INTERRUPTS_BLOCKED_SMI_BLOCKED (1 << 2)
@@ -689,7 +781,8 @@ enum VMX_Activity_State {
 
 #define VMX_MSR_VMX_BASIC_LO (VMX_VMCS_REVISION_ID)
 #define VMX_MSR_VMX_BASIC_HI \
-     (VMX_VMCS_AREA_SIZE | ((!BX_SUPPORT_X86_64) << 16) | (BX_MEMTYPE_WB << 18))
+     (VMX_VMCS_AREA_SIZE | ((!BX_SUPPORT_X86_64) << 16) | \
+     (BX_MEMTYPE_WB << 18) | (1<<22)) | ((BX_SUPPORT_VMX >= 2) ? (1<<23) : 0)
 
 #define VMX_MSR_VMX_BASIC \
    ((((Bit64u) VMX_MSR_VMX_BASIC_HI) << 32) | VMX_MSR_VMX_BASIC_LO)
@@ -716,7 +809,7 @@ enum VMX_Activity_State {
 
 // Allowed 0-settings: VMentry fail if a bit is '0 in pin-based vmexec controls
 // but set to '1 in this MSR
-#define VMX_MSR_VMX_PINBASED_CTRLS_LO  (0x00000016)
+#define VMX_MSR_VMX_PINBASED_CTRLS_LO (0x00000016)
 // Allowed 1-settings: VMentry fail if a bit is '1 in pin-based vmexec controls
 // but set to '0 in this MSR.
 #define VMX_MSR_VMX_PINBASED_CTRLS_HI \
@@ -725,6 +818,16 @@ enum VMX_Activity_State {
 #define VMX_MSR_VMX_PINBASED_CTRLS \
    ((((Bit64u) VMX_MSR_VMX_PINBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_PINBASED_CTRLS_LO)
 
+// IA32_MSR_VMX_TRUE_PINBASED_CTRLS MSR (0x48d)
+// --------------------------------
+
+// no changes from original IA32_MSR_VMX_PINBASED_CTRLS
+#define VMX_MSR_VMX_TRUE_PINBASED_CTRLS_LO (VMX_MSR_VMX_PINBASED_CTRLS_LO)
+#define VMX_MSR_VMX_TRUE_PINBASED_CTRLS_HI (VMX_MSR_VMX_PINBASED_CTRLS_HI)
+
+#define VMX_MSR_VMX_TRUE_PINBASED_CTRLS \
+   ((((Bit64u) VMX_MSR_VMX_TRUE_PINBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_PINBASED_CTRLS_LO)
+
 
 // IA32_MSR_VMX_PROCBASED_CTRLS MSR (0x482)
 // ----------------------------
@@ -742,6 +845,16 @@ enum VMX_Activity_State {
 #define VMX_MSR_VMX_PROCBASED_CTRLS \
    ((((Bit64u) VMX_MSR_VMX_PROCBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_PROCBASED_CTRLS_LO)
 
+// IA32_MSR_VMX_TRUE_PROCBASED_CTRLS MSR (0x48e)
+// ---------------------------------
+
+// Bits 15 and 16 no longer must be '1
+#define VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_LO (0x04006172)
+#define VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_HI (VMX_MSR_VMX_PROCBASED_CTRLS_HI)
+
+#define VMX_MSR_VMX_TRUE_PROCBASED_CTRLS \
+   ((((Bit64u) VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_LO)
+
 
 // IA32_MSR_VMX_VMEXIT_CTRLS MSR (0x483)
 // -------------------------
@@ -749,7 +862,7 @@ enum VMX_Activity_State {
 // Bits 0-8, 10, 11, 13, 14, 16, 17 must be '1
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_VMEXIT_CTRLS_LO    (0x00036DFF)
+#define VMX_MSR_VMX_VMEXIT_CTRLS_LO (0x00036DFF)
 // Allowed 1-settings
 #define VMX_MSR_VMX_VMEXIT_CTRLS_HI \
        (VMX_VMEXIT_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_VMEXIT_CTRLS_LO)
@@ -757,6 +870,16 @@ enum VMX_Activity_State {
 #define VMX_MSR_VMX_VMEXIT_CTRLS \
    ((((Bit64u) VMX_MSR_VMX_VMEXIT_CTRLS_HI) << 32) | VMX_MSR_VMX_VMEXIT_CTRLS_LO)
 
+// IA32_MSR_VMX_TRUE_VMEXIT_CTRLS MSR (0x48f)
+// ------------------------------
+
+// Bit 2 no longer must be '1
+#define VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_LO (0x00036DFB)
+#define VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_HI (VMX_MSR_VMX_VMEXIT_CTRLS_HI)
+
+#define VMX_MSR_VMX_TRUE_VMEXIT_CTRLS \
+   ((((Bit64u) VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_LO)
+
 
 // IA32_MSR_VMX_VMENTRY_CTRLS MSR (0x484)
 // --------------------------
@@ -764,14 +887,46 @@ enum VMX_Activity_State {
 // Bits 0-8, 12 must be '1
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_VMENTRY_CTRLS_LO   (0x000011FF)
+#define VMX_MSR_VMX_VMENTRY_CTRLS_LO (0x000011FF)
 // Allowed 1-settings
 #define VMX_MSR_VMX_VMENTRY_CTRLS_HI \
-       (VMX_VMEXIT_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_VMENTRY_CTRLS_LO)
+       (VMX_VMENTRY_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_VMENTRY_CTRLS_LO)
 
 #define VMX_MSR_VMX_VMENTRY_CTRLS \
    ((((Bit64u) VMX_MSR_VMX_VMENTRY_CTRLS_HI) << 32) | VMX_MSR_VMX_VMENTRY_CTRLS_LO)
 
+// IA32_MSR_VMX_TRUE_VMENTRY_CTRLS MSR (0x490)
+// -------------------------------
+
+// Bit 2 is longer must be '1
+#define VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_LO (0x000011FB)
+#define VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_HI (VMX_MSR_VMX_VMENTRY_CTRLS_HI)
+
+#define VMX_MSR_VMX_TRUE_VMENTRY_CTRLS \
+   ((((Bit64u) VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_LO)
+
+
+// IA32_MSR_VMX_MISC MSR (0x485)
+// -----------------
+
+//   [4:0] - TSC:VMX_PREEMPTION_TIMER ratio
+//     [5] - VMEXITs store the value of EFER.LMA into the x86-64 guest"
+//           VMENTRY control (must set to '1 if 'unrestricted guest' is supported)
+//     [6] - support VMENTER to HLT state
+//     [7] - support VMENTER to SHUTDOWN state
+//     [8] - support VMENTER to WAIT_FOR_SIPI state
+// [24:16] - number of CR3 target values supported
+// [27:25] - (N+1)*512 - recommended maximum MSRs in MSR store list
+// [63:32] - MSEG revision ID used by processor
+
+#if BX_SUPPORT_VMX >= 2
+  #define VMX_MISC_STORE_LMA_TO_X86_64_GUEST_VMENTRY_CONTROL (1<<5)
+#else
+  #define VMX_MISC_STORE_LMA_TO_X86_64_GUEST_VMENTRY_CONTROL (0)
+#endif
+
+#define VMX_MSR_MISC ((VMX_CR3_TARGET_MAX_CNT << 16) | \
+            VMX_MISC_STORE_LMA_TO_X86_64_GUEST_VMENTRY_CONTROL)
 
 //
 // IA32_VMX_CR0_FIXED0 MSR (0x486)   IA32_VMX_CR0_FIXED1 MSR (0x487)
@@ -779,42 +934,41 @@ enum VMX_Activity_State {
 
 // allowed 0-setting in CR0 in VMX mode
 // bits PE(0), NE(5) and PG(31) required to be set in CR0 to enter VMX mode
-#define VMX_MSR_CR0_FIXED0_LO          (0x80000021)
-#define VMX_MSR_CR0_FIXED0_HI          (0x00000000)
+#define VMX_MSR_CR0_FIXED0_LO (0x80000021)
+#define VMX_MSR_CR0_FIXED0_HI (0x00000000)
 
 #define VMX_MSR_CR0_FIXED0 \
    ((((Bit64u) VMX_MSR_CR0_FIXED0_HI) << 32) | VMX_MSR_CR0_FIXED0_LO)
 
 // allowed 1-setting in CR0 in VMX mode
-#define VMX_MSR_CR0_FIXED1_LO          (0xFFFFFFFF)
-#define VMX_MSR_CR0_FIXED1_HI          (0x00000000)
+#define VMX_MSR_CR0_FIXED1_LO (0xFFFFFFFF)
+#define VMX_MSR_CR0_FIXED1_HI (0x00000000)
 
 #define VMX_MSR_CR0_FIXED1 \
    ((((Bit64u) VMX_MSR_CR0_FIXED1_HI) << 32) | VMX_MSR_CR0_FIXED1_LO)
 
-
 //
 // IA32_VMX_CR4_FIXED0 MSR (0x488)   IA32_VMX_CR4_FIXED1 MSR (0x489)
 // -------------------               -------------------
 
 // allowed 0-setting in CR0 in VMX mode
 // bit VMXE(13) required to be set in CR4 to enter VMX mode
-#define VMX_MSR_CR4_FIXED0_LO          (0x00002000)
-#define VMX_MSR_CR4_FIXED0_HI          (0x00000000)
+#define VMX_MSR_CR4_FIXED0_LO (0x00002000)
+#define VMX_MSR_CR4_FIXED0_HI (0x00000000)
 
 #define VMX_MSR_CR4_FIXED0 \
    ((((Bit64u) VMX_MSR_CR4_FIXED0_HI) << 32) | VMX_MSR_CR4_FIXED0_LO)
 
 // allowed 1-setting in CR0 in VMX mode
-#define VMX_MSR_CR4_FIXED1_LO GET32L(get_cr4_allow_mask())
-#define VMX_MSR_CR4_FIXED1_HI GET32H(get_cr4_allow_mask())
+#define VMX_MSR_CR4_FIXED1_LO GET32L(get_cr4_allow_mask(BX_CPU_THIS_PTR isa_extensions_bitmask))
+#define VMX_MSR_CR4_FIXED1_HI GET32H(get_cr4_allow_mask(BX_CPU_THIS_PTR isa_extensions_bitmask))
 
 #define VMX_MSR_CR4_FIXED1 \
    ((((Bit64u) VMX_MSR_CR4_FIXED1_HI) << 32) | VMX_MSR_CR4_FIXED1_LO)
 
 
 //
-// IA32_VMX_VMCS_ENUM MSR (0x48A)
+// IA32_VMX_VMCS_ENUM MSR (0x48a)
 // ------------------
 
 //
@@ -822,7 +976,7 @@ enum VMX_Activity_State {
 // 63:10 reserved, must be zero
 //
 
-#define VMX_HIGHEST_VMCS_ENCODING 0x2A
+#define VMX_HIGHEST_VMCS_ENCODING 0x2C
 
 #define VMX_MSR_VMCS_ENUM_LO (VMX_HIGHEST_VMCS_ENCODING)
 #define VMX_MSR_VMCS_ENUM_HI (0x00000000)
@@ -831,4 +985,54 @@ enum VMX_Activity_State {
    ((((Bit64u) VMX_MSR_VMCS_ENUM_HI) << 32) | VMX_MSR_VMCS_ENUM_LO)
 
 
+#if BX_SUPPORT_VMX >= 2
+
+// IA32_VMX_MSR_PROCBASED_CTRLS2 MSR (0x48b)
+// -----------------------------
+
+// Allowed 0-settings (must be '1 bits)
+#define VMX_MSR_VMX_PROCBASED_CTRLS2_LO (0x00000000)
+// Allowed 1-settings
+#define VMX_MSR_VMX_PROCBASED_CTRLS2_HI \
+       (VMX_VM_EXEC_CTRL3_SUPPORTED_BITS | VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
+
+#define VMX_MSR_VMX_PROCBASED_CTRLS2 \
+   ((((Bit64u) VMX_MSR_VMX_PROCBASED_CTRLS2_HI) << 32) | VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
+
+
+// IA32_VMX_EPT_VPID_CAP MSR (0x48c)
+// ---------------------
+
+enum VMX_INVEPT_INVVPID_type {
+  BX_INVEPT_INVVPID_INDIVIDUAL_ADDRESS_INVALIDATION = 0,
+  BX_INVEPT_INVVPID_SINGLE_CONTEXT_INVALIDATION,
+  BX_INVEPT_INVVPID_ALL_CONTEXT_INVALIDATION,
+  BX_INVEPT_INVVPID_SINGLE_CONTEXT_NON_GLOBAL_INVALIDATION
+};
+
+//  [0] - BX_EPT_ENTRY_EXECUTE_ONLY support
+//  [6] - 4-levels page walk length
+//  [8] - allow UC EPT paging structure memory type
+// [14] - allow WB EPT paging structure memory type
+// [16] - EPT 2M pages support
+// [17] - EPT 1G pages support
+// [20] - INVEPT instruction supported
+// [25] - INVEPT single-context invalidation supported
+// [26] - INVEPT all-context invalidation supported
+
+#define VMX_MSR_VMX_EPT_VPID_CAP_LO (0x06114141 | (bx_cpuid_support_1g_paging() << 17))
+
+// [32] - INVVPID instruction supported
+// [40] - individual-address INVVPID is supported
+// [41] - single-context INVVPID is supported
+// [42] - all-context INVVPID is supported
+// [43] - single-context-retaining-globals INVVPID is supported
+
+#define VMX_MSR_VMX_EPT_VPID_CAP_HI (0x00000f01)
+
+#define VMX_MSR_VMX_EPT_VPID_CAP \
+   ((((Bit64u) VMX_MSR_VMX_EPT_VPID_CAP_HI) << 32) | VMX_MSR_VMX_EPT_VPID_CAP_LO)
+
+#endif
+
 #endif // _BX_VMX_INTEL_H_
