@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gui.cc,v 1.112 2009/04/20 18:11:13 vruppert Exp $
+// $Id: gui.cc,v 1.114 2010/02/26 14:18:18 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2002-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +20,7 @@
 
 
 #include <signal.h>
+#include "param_names.h"
 #include "bochs.h"
 #include "keymap.h"
 #include "iodev.h"
