@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: x.cc,v 1.127 2009/10/24 13:22:36 sshwarts Exp $
+// $Id: x.cc,v 1.130 2010/02/26 14:18:18 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2002-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,6 +27,7 @@
 #define BX_PLUGGABLE
 
 #include "bochs.h"
+#include "param_names.h"
 #include "keymap.h"
 #include "iodev.h"
 #include "enh_dbg.h"
@@ -133,12 +128,12 @@ static bx_bool x_init_done = false;
 
 static Pixmap vgafont[256];
 
+static unsigned bx_bitmap_entries = 0;
 static struct {
   Pixmap bmap;
   unsigned xdim;
   unsigned ydim;
-  } bx_bitmaps[BX_MAX_PIXMAPS];
-unsigned bx_bitmap_entries = 0;
+} bx_bitmaps[BX_MAX_PIXMAPS];
 
 static struct {
   Pixmap   bitmap;
@@ -148,7 +143,8 @@ static struct {
   unsigned yorigin;
   unsigned alignment;
   void (*f)(void);
-  } bx_headerbar_entry[BX_MAX_HEADERBAR_ENTRIES];
+} bx_headerbar_entry[BX_MAX_HEADERBAR_ENTRIES];
+
 static unsigned bx_headerbar_y = 0;
 static unsigned bx_headerbar_entries = 0;
 static unsigned bx_bitmap_left_xorigin = 0;  // pixels from left
@@ -157,7 +153,7 @@ static unsigned bx_bitmap_right_xorigin = 0; // pixels from right
 static unsigned bx_statusbar_y = 18;
 static unsigned bx_statusitem_pos[12] = {
   0, 200, 240, 280, 320, 360, 400, 440, 480, 520, 560, 600
-  };
+};
 static bx_bool bx_statusitem_active[12];
 static long bx_status_led_green, bx_status_led_red, bx_status_graytext;
 static char bx_status_info_text[34];
