@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: devices.cc,v 1.148 2009/10/15 16:14:30 sshwarts Exp $
+// $Id: devices.cc,v 1.150 2009/12/04 19:50:26 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2002-2009  The Bochs Project
 //
 //  I/O port handlers API Copyright (C) 2003 by Frank Cornelis
 //
@@ -92,7 +86,7 @@ void bx_devices_c::init_stubs()
 #if BX_SUPPORT_IODEBUG
   pluginIODebug = &stubIODebug;
 #endif
-#if BX_SUPPORT_IOAPIC
+#if BX_SUPPORT_APIC
   pluginIOAPIC = &stubIOAPIC;
 #endif
 #if 0
@@ -110,7 +104,7 @@ void bx_devices_c::init(BX_MEM_C *newmem)
   const char *plugname;
 #endif
 
-  BX_DEBUG(("Init $Id: devices.cc,v 1.148 2009/10/15 16:14:30 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: devices.cc,v 1.150 2009/12/04 19:50:26 sshwarts Exp $"));
   mem = newmem;
 
   /* set builtin default handlers, will be overwritten by the real default handler */
