@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_vde.cc,v 1.15 2008/02/15 22:05:42 sshwarts Exp $
+// $Id: eth_vde.cc,v 1.20 2009/04/19 17:25:40 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  Renzo Davoli
@@ -16,7 +16,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
@@ -35,7 +35,7 @@
 
 #include "eth.h"
 
-#define LOG_THIS bx_devices.pluginNE2kDevice->
+#define LOG_THIS netdev->
 
 #include <signal.h>
 #include <sys/param.h>
@@ -72,14 +72,14 @@ int vde_alloc(char *dev,int *fdp,struct sockaddr_un *pdataout);
 class bx_vde_pktmover_c : public eth_pktmover_c {
 public:
   bx_vde_pktmover_c(const char *netif, const char *macaddr,
-		     eth_rx_handler_t rxh,
-		     void *rxarg, char *script);
+                    eth_rx_handler_t rxh,
+                    bx_devmodel_c *dev, const char *script);
   void sendpkt(void *buf, unsigned io_len);
 private:
   int fd;
   int rx_timer_index;
   static void rx_timer_handler(void *);
-  void rx_timer ();
+  void rx_timer();
   FILE *txlog, *txlog_txt, *rxlog, *rxlog_txt;
   int fddata;
   struct sockaddr_un dataout;
@@ -95,9 +95,9 @@ public:
   bx_vde_locator_c(void) : eth_locator_c("vde") {}
 protected:
   eth_pktmover_c *allocate(const char *netif, const char *macaddr,
-			   eth_rx_handler_t rxh,
-			   void *rxarg, char *script) {
-    return (new bx_vde_pktmover_c(netif, macaddr, rxh, rxarg, script));
+                           eth_rx_handler_t rxh,
+                           bx_devmodel_c *dev, const char *script) {
+    return (new bx_vde_pktmover_c(netif, macaddr, rxh, dev, script));
   }
 } bx_vde_match;
 
@@ -108,33 +108,35 @@ protected:
 
 // the constructor
 bx_vde_pktmover_c::bx_vde_pktmover_c(const char *netif,
-				       const char *macaddr,
-				       eth_rx_handler_t rxh,
-				       void *rxarg,
-				       char *script)
+                                     const char *macaddr,
+                                     eth_rx_handler_t rxh,
+                                     bx_devmodel_c *dev,
+                                     const char *script)
 {
   int flags;
+
+  this->netdev = dev;
   //if (strncmp (netif, "vde", 3) != 0) {
    // BX_PANIC (("eth_vde: interface name (%s) must be vde", netif));
   //}
   char intname[IFNAMSIZ];
   if (netif == NULL || strcmp(netif,"") == 0)
-	  strcpy(intname,"/tmp/vde.ctl");
+    strcpy(intname,"/tmp/vde.ctl");
   else
-  	strcpy(intname,netif);
+    strcpy(intname,netif);
   fd=vde_alloc(intname,&fddata,&dataout);
   if (fd < 0) {
-    BX_PANIC (("open failed on %s: %s", netif, strerror (errno)));
+    BX_PANIC(("open failed on %s: %s", netif, strerror (errno)));
     return;
   }
 
   /* set O_ASYNC flag so that we can poll with read() */
   if ((flags = fcntl(fd, F_GETFL)) < 0) {
-    BX_PANIC (("getflags on vde device: %s", strerror (errno)));
+    BX_PANIC(("getflags on vde device: %s", strerror (errno)));
   }
   flags |= O_NONBLOCK;
   if (fcntl(fd, F_SETFL, flags) < 0) {
-    BX_PANIC (("set vde device flags: %s", strerror (errno)));
+    BX_PANIC(("set vde device flags: %s", strerror (errno)));
   }
 
   BX_INFO(("eth_vde: opened %s device", netif));
@@ -142,42 +144,41 @@ bx_vde_pktmover_c::bx_vde_pktmover_c(const char *netif,
   /* Execute the configuration script */
   if((script != NULL) && (strcmp(script, "") != 0) && (strcmp(script, "none") != 0))
   {
-    if (execute_script(script, intname) < 0)
-      BX_ERROR (("execute script '%s' on %s failed", script, intname));
+    if (execute_script(this->netdev, script, intname) < 0)
+      BX_ERROR(("execute script '%s' on %s failed", script, intname));
   }
 
   // Start the rx poll
   this->rx_timer_index =
     bx_pc_system.register_timer(this, this->rx_timer_handler, 1000,
-				1, 1, "eth_vde"); // continuous, active
+                                1, 1, "eth_vde"); // continuous, active
   this->rxh   = rxh;
-  this->rxarg = rxarg;
 #if BX_ETH_VDE_LOGGING
   // eventually Bryce wants txlog to dump in pcap format so that
   // tcpdump -r FILE can read it and interpret packets.
-  txlog = fopen ("ne2k-tx.log", "wb");
-  if (!txlog) BX_PANIC (("open ne2k-tx.log failed"));
-  txlog_txt = fopen ("ne2k-txdump.txt", "wb");
-  if (!txlog_txt) BX_PANIC (("open ne2k-txdump.txt failed"));
-  fprintf (txlog_txt, "vde packetmover readable log file\n");
-  fprintf (txlog_txt, "net IF = %s\n", netif);
-  fprintf (txlog_txt, "MAC address = ");
+  txlog = fopen("ne2k-tx.log", "wb");
+  if (!txlog) BX_PANIC(("open ne2k-tx.log failed"));
+  txlog_txt = fopen("ne2k-txdump.txt", "wb");
+  if (!txlog_txt) BX_PANIC(("open ne2k-txdump.txt failed"));
+  fprintf(txlog_txt, "vde packetmover readable log file\n");
+  fprintf(txlog_txt, "net IF = %s\n", netif);
+  fprintf(txlog_txt, "MAC address = ");
   for (int i=0; i<6; i++)
-    fprintf (txlog_txt, "%02x%s", 0xff & macaddr[i], i<5?":" : "");
-  fprintf (txlog_txt, "\n--\n");
-  fflush (txlog_txt);
+    fprintf(txlog_txt, "%02x%s", 0xff & macaddr[i], i<5?":" : "");
+  fprintf(txlog_txt, "\n--\n");
+  fflush(txlog_txt);
 
-  rxlog = fopen ("ne2k-rx.log", "wb");
-  if (!rxlog) BX_PANIC (("open ne2k-rx.log failed"));
-  rxlog_txt = fopen ("ne2k-rxdump.txt", "wb");
-  if (!rxlog_txt) BX_PANIC (("open ne2k-rxdump.txt failed"));
-  fprintf (rxlog_txt, "vde packetmover readable log file\n");
-  fprintf (rxlog_txt, "net IF = %s\n", netif);
-  fprintf (rxlog_txt, "MAC address = ");
+  rxlog = fopen("ne2k-rx.log", "wb");
+  if (!rxlog) BX_PANIC(("open ne2k-rx.log failed"));
+  rxlog_txt = fopen("ne2k-rxdump.txt", "wb");
+  if (!rxlog_txt) BX_PANIC(("open ne2k-rxdump.txt failed"));
+  fprintf(rxlog_txt, "vde packetmover readable log file\n");
+  fprintf(rxlog_txt, "net IF = %s\n", netif);
+  fprintf(rxlog_txt, "MAC address = ");
   for (int i=0; i<6; i++)
-    fprintf (rxlog_txt, "%02x%s", 0xff & macaddr[i], i<5?":" : "");
-  fprintf (rxlog_txt, "\n--\n");
-  fflush (rxlog_txt);
+    fprintf(rxlog_txt, "%02x%s", 0xff & macaddr[i], i<5?":" : "");
+  fprintf(rxlog_txt, "\n--\n");
+  fflush(rxlog_txt);
 
 #endif
 }
@@ -189,38 +190,30 @@ void bx_vde_pktmover_c::sendpkt(void *buf, unsigned io_len)
   //size=send(fd,buf,io_len,0);
   size=sendto(fddata,buf,io_len,0,(struct sockaddr *) &dataout, sizeof(struct sockaddr_un));
   if (size != io_len) {
-    BX_PANIC (("write on vde device: %s", strerror (errno)));
+    BX_PANIC(("write on vde device: %s", strerror (errno)));
   } else {
-    BX_INFO (("wrote %d bytes on vde", io_len));
+    BX_INFO(("wrote %d bytes on vde", io_len));
   }
 #if BX_ETH_VDE_LOGGING
-  BX_DEBUG (("sendpkt length %u", io_len));
+  BX_DEBUG(("sendpkt length %u", io_len));
   // dump raw bytes to a file, eventually dump in pcap format so that
   // tcpdump -r FILE can interpret them for us.
-  int n = fwrite (buf, io_len, 1, txlog);
-  if (n != 1) BX_ERROR (("fwrite to txlog failed", io_len));
+  int n = fwrite(buf, io_len, 1, txlog);
+  if (n != 1) BX_ERROR(("fwrite to txlog failed"));
   // dump packet in hex into an ascii log file
-  fprintf (txlog_txt, "NE2K transmitting a packet, length %u\n", io_len);
-  Bit8u *charbuf = (Bit8u *)buf;
-  for (n=0; n<io_len; n++) {
-    if (((n % 16) == 0) && n>0)
-      fprintf (txlog_txt, "\n");
-    fprintf (txlog_txt, "%02x ", charbuf[n]);
-  }
-  fprintf (txlog_txt, "\n--\n");
+  write_pktlog_txt(txlog_txt, (const Bit8u *)buf, io_len, 0);
   // flush log so that we see the packets as they arrive w/o buffering
-  fflush (txlog);
-  fflush (txlog_txt);
+  fflush(txlog);
 #endif
 }
 
-void bx_vde_pktmover_c::rx_timer_handler (void *this_ptr)
+void bx_vde_pktmover_c::rx_timer_handler(void *this_ptr)
 {
   bx_vde_pktmover_c *class_ptr = (bx_vde_pktmover_c *) this_ptr;
   class_ptr->rx_timer();
 }
 
-void bx_vde_pktmover_c::rx_timer ()
+void bx_vde_pktmover_c::rx_timer()
 {
   int nbytes;
   Bit8u buf[BX_PACKET_BUFSIZE];
@@ -235,38 +228,32 @@ void bx_vde_pktmover_c::rx_timer ()
   rxbuf=buf;
 
   if (nbytes>0)
-    BX_INFO (("vde read returned %d bytes", nbytes));
+    BX_INFO(("vde read returned %d bytes", nbytes));
   if (nbytes<0) {
     if (errno != EAGAIN)
-      BX_ERROR (("vde read error: %s", strerror(errno)));
+      BX_ERROR(("vde read error: %s", strerror(errno)));
     return;
   }
 #if BX_ETH_VDE_LOGGING
   if (nbytes > 0) {
-    BX_DEBUG (("receive packet length %u", nbytes));
+    BX_DEBUG(("receive packet length %u", nbytes));
     // dump raw bytes to a file, eventually dump in pcap format so that
     // tcpdump -r FILE can interpret them for us.
-    int n = fwrite (rxbuf, nbytes, 1, rxlog);
-    if (n != 1) BX_ERROR (("fwrite to rxlog failed", nbytes));
+    int n = fwrite(rxbuf, nbytes, 1, rxlog);
+    if (n != 1) BX_ERROR(("fwrite to rxlog failed"));
     // dump packet in hex into an ascii log file
-    fprintf (rxlog_txt, "NE2K received a packet, length %u\n", nbytes);
-    for (n=0; n<nbytes; n++) {
-      if (((n % 16) == 0) && n>0)
-	fprintf (rxlog_txt, "\n");
-      fprintf (rxlog_txt, "%02x ", rxbuf[n]);
-    }
-    fprintf (rxlog_txt, "\n--\n");
+    write_pktlog_txt(rxlog_txt, rxbuf, nbytes, 1);
+
     // flush log so that we see the packets as they arrive w/o buffering
-    fflush (rxlog);
-    fflush (rxlog_txt);
+    fflush(rxlog);
   }
 #endif
   BX_DEBUG(("eth_vde: got packet: %d bytes, dst=%x:%x:%x:%x:%x:%x, src=%x:%x:%x:%x:%x:%x\n", nbytes, rxbuf[0], rxbuf[1], rxbuf[2], rxbuf[3], rxbuf[4], rxbuf[5], rxbuf[6], rxbuf[7], rxbuf[8], rxbuf[9], rxbuf[10], rxbuf[11]));
   if (nbytes < 60) {
-    BX_INFO (("packet too short (%d), padding to 60", nbytes));
+    BX_INFO(("packet too short (%d), padding to 60", nbytes));
     nbytes = 60;
   }
-  (*rxh)(rxarg, rxbuf, nbytes);
+  (*rxh)(this->netdev, rxbuf, nbytes);
 }
 
 //enum request_type { REQ_NEW_CONTROL };
