@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: serial_raw.cc,v 1.22 2008/01/26 22:24:02 sshwarts Exp $
+// $Id: serial_raw.cc,v 1.25 2009/02/08 09:05:52 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2004  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2004  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +16,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
@@ -53,7 +47,6 @@ serial_raw::serial_raw(const char *devname)
 #endif
 
   put ("SERR");
-  settype (SERRLOG);
 #ifdef WIN32
   memset(&dcb, 0, sizeof(DCB));
   dcb.DCBlength = sizeof(DCB);
