@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.cc,v 1.80 2009/10/17 18:42:15 sshwarts Exp $
+// $Id: memory.cc,v 1.82 2009/12/04 16:53:12 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2009  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -234,7 +228,7 @@ void BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, bx_phy_address addr, unsigned len
 
 mem_read:
 
-  if (a20addr <= BX_MEM_THIS len && ! is_bios) {
+  if (a20addr < BX_MEM_THIS len && ! is_bios) {
     // all of data is within limits of physical memory
     if (a20addr < 0x000a0000 || a20addr >= 0x00100000)
     {
